@@ -1,12 +1,10 @@
 open Import
 
-let pp_with_ocamlc sctx project pp_file =
+let pp_with_ocamlc env ~ocamlc dialects pp_file =
   let open Dune_engine in
   let dump_file =
     Path.map_extension pp_file ~f:(fun ext ->
-      let dialect =
-        Dune_rules.Dialect.DB.find_by_extension (Dune_project.dialects project) ext
-      in
+      let dialect = Dune_rules.Dialect.DB.find_by_extension dialects ext in
       match dialect with
       | None -> User_error.raise [ Pp.textf "unsupported extension: %s" ext ]
       | Some (_, (kind : Ocaml.Ml_kind.t)) ->
@@ -18,9 +16,9 @@ let pp_with_ocamlc sctx project pp_file =
   let+ () =
     Process.run
       ~display:!Clflags.display
-      ~env:(Super_context.context_env sctx)
+      ~env
       Strict
-      (Super_context.context sctx |> Context.ocaml).ocamlc
+      ocamlc
       [ "-stop-after"; "parsing"; "-dsource"; Path.to_string pp_file; "-dump-into-file" ]
   in
   match Path.stat dump_file with
@@ -52,45 +50,44 @@ let get_pped_file super_context file =
   | false ->
     Build_system.file_exists file_in_build_dir
     >>= (function
-    | true ->
-      let* dir =
-        Source_tree.nearest_dir (Path.Source.of_string file)
-        >>| Source_tree.Dir.path
-        >>| Path.source
-      in
-      let* dune_file = Dune_rules.Dune_load.Dune_files.in_dir (dir |> in_build_dir) in
-      let staged_pps =
-        Option.bind dune_file ~f:(fun dune_file ->
-          dune_file.stanzas
-          |> List.fold_left ~init:None ~f:(fun acc stanza ->
-            match stanza with
-            | Dune_rules.Dune_file.Library lib ->
-              let preprocess =
-                Dune_rules.Preprocess.Per_module.(
-                  lib.buildable.preprocess |> single_preprocess)
-              in
-              (match preprocess with
-               | Dune_rules.Preprocess.Pps ({ staged = true; _ } as pps) -> Some pps
-               | _ -> acc)
-            | _ -> acc))
-      in
-      (match staged_pps with
-       | None ->
-         let+ () = Build_system.build_file file_in_build_dir in
-         Error file_in_build_dir
-       | Some { loc; _ } ->
-         User_error.raise ~loc [ Pp.text "staged_pps are not supported." ])
-    | false ->
-      User_error.raise
-        [ Pp.textf "%s does not exist" (Path.to_string_maybe_quoted file_in_build_dir) ])
+     | true ->
+       let* dir =
+         Source_tree.nearest_dir (Path.Source.of_string file)
+         >>| Source_tree.Dir.path
+         >>| Path.source
+       in
+       let* dune_file = Dune_rules.Dune_load.stanzas_in_dir (dir |> in_build_dir) in
+       let* staged_pps =
+         match dune_file with
+         | None -> Memo.return None
+         | Some dune_file ->
+           Dune_file.find_stanzas dune_file Dune_rules.Library.key
+           >>| List.fold_left ~init:None ~f:(fun acc (lib : Dune_rules.Library.t) ->
+             let preprocess =
+               Dune_rules.Preprocess.Per_module.(
+                 lib.buildable.preprocess |> single_preprocess)
+             in
+             match preprocess with
+             | Dune_rules.Preprocess.Pps ({ staged = true; _ } as pps) -> Some pps
+             | _ -> acc)
+       in
+       (match staged_pps with
+        | None ->
+          let+ () = Build_system.build_file file_in_build_dir in
+          Error file_in_build_dir
+        | Some { loc; _ } ->
+          User_error.raise ~loc [ Pp.text "staged_pps are not supported." ])
+     | false ->
+       User_error.raise
+         [ Pp.textf "%s does not exist" (Path.to_string_maybe_quoted file_in_build_dir) ])
 ;;
 
 let term =
-  let+ common = Common.term
+  let+ builder = Common.Builder.term
   and+ context_name = Common.context_arg ~doc:"Build context to use."
   and+ _ = Describe_lang_compat.arg
   and+ file = Arg.(required & pos 0 (some string) None (Arg.info [] ~docv:"FILE")) in
-  let config = Common.init common in
+  let common, config = Common.init builder in
   Scheduler.go ~common ~config
   @@ fun () ->
   let open Fiber.O in
@@ -104,7 +101,13 @@ let term =
   match result with
   | Error file -> Io.cat file |> Memo.return
   | Ok (project, file) ->
-    pp_with_ocamlc super_context project file |> Memo.of_non_reproducible_fiber
+    let* ocamlc =
+      let+ ocaml = Context.ocaml (Super_context.context super_context) in
+      ocaml.ocamlc
+    in
+    let env = Super_context.context_env super_context in
+    let dialects = Dune_project.dialects project in
+    pp_with_ocamlc env ~ocamlc dialects file |> Memo.of_non_reproducible_fiber
 ;;
 
 let command =
