@@ -1,10 +1,10 @@
 open Import
 
 let term =
-  let+ common = Common.term
+  let+ builder = Common.Builder.term
   and+ context_name = Common.context_arg ~doc:"Build context to use."
   and+ format = Describe_format.arg in
-  let config = Common.init common in
+  let common, config = Common.init builder in
   Scheduler.go ~common ~config
   @@ fun () ->
   let open Fiber.O in
@@ -15,8 +15,7 @@ let term =
   @@ fun () ->
   let open Memo.O in
   Dune_rules.Install_rules.stanzas_to_entries super_context
-  >>| Package.Name.Map.to_dyn (fun entries ->
-    Dyn.List (List.map ~f:Install.Entry.Sourced.to_dyn entries))
+  >>| Package.Name.Map.to_dyn (Dyn.list Install.Entry.Sourced.to_dyn)
   >>| Describe_format.print_dyn format
 ;;
 
