@@ -4,7 +4,7 @@ let doc = "Print out libraries installed on the system."
 let info = Cmd.info "installed-libraries" ~doc
 
 let term =
-  let+ common = Common.term
+  let+ builder = Common.Builder.term
   and+ na =
     Arg.(
       value
@@ -13,7 +13,7 @@ let term =
           [ "na"; "not-available" ]
           ~doc:"List libraries that are not available and explain why")
   in
-  let config = Common.init common in
+  let common, config = Common.init builder in
   Scheduler.go
     ~common
     ~config
@@ -58,16 +58,16 @@ let term =
              Lib_name.to_string (Dune_package.Entry.name e))
          in
          List.iter pkgs ~f:(fun e ->
-           let ver =
+           let ver_string =
              match Dune_package.Entry.version e with
-             | Some v -> v
+             | Some v -> Package_version.to_string v
              | _ -> "n/a"
            in
            Printf.printf
              "%-*s (version: %s)\n"
              max_len
              (Lib_name.to_string (Dune_package.Entry.name e))
-             ver);
+             ver_string);
          Memo.return ())
      in
      fun () -> Memo.run (run ()))
