@@ -9,7 +9,7 @@ let man =
   ; `P
       {|Print a list of toplevel directives for including directories and loading cma files.|}
   ; `P
-      {|The output of $(b,dune toplevel-init-file) should be evaluated in a toplevel
+      {|The output of $(b,dune top) should be evaluated in a toplevel
           to make a library available there.|}
   ; `Blocks Common.help_secs
   ]
@@ -34,10 +34,10 @@ let files_to_load_of_requires sctx requires =
 ;;
 
 let term =
-  let+ common = Common.term
+  let+ builder = Common.Builder.term
   and+ dir = Arg.(value & pos 0 string "" & Arg.info [] ~docv:"DIR")
   and+ ctx_name = Common.context_arg ~doc:{|Select context where to build/run utop.|} in
-  let config = Common.init common in
+  let common, config = Common.init builder in
   Scheduler.go ~common ~config (fun () ->
     let open Fiber.O in
     let* setup = Import.Main.setup () in
@@ -80,7 +80,7 @@ module Module = struct
         "The module's source is evaluated in the toplevel without being sealed by the \
          mli."
     ; `P
-        {|The output of $(b,dune toplevel-init-file) should be evaluated in a toplevel
+        {|The output of $(b,dune top) should be evaluated in a toplevel
           to make the module available there.|}
     ; `Blocks Common.help_secs
     ]
@@ -97,7 +97,7 @@ module Module = struct
     then User_error.raise [ Pp.text "file is missing an extension" ];
     let open Memo.O in
     let module_name =
-      let name = Filename.chop_extension filename in
+      let name = Filename.remove_extension filename in
       Dune_rules.Module_name.of_string_user_error (Loc.none, name) |> User_error.ok_exn
     in
     let* expander = Super_context.expander sctx ~dir in
@@ -128,8 +128,8 @@ module Module = struct
                     ~dir:(Path.build (Obj_dir.byte_dir private_obj_dir))
                     (Dune_lang.Glob.of_string_exn Loc.none "*.cmi")
                 in
-                let+ (_ : Dep.Fact.Files.t) = Build_system.build_pred glob in
-                ()
+                let* files = Build_system.eval_pred glob in
+                Memo.parallel_iter (Filename_set.to_list files) ~f:Build_system.build_file
               in
               let cmos () =
                 let obj_dir = Compilation_context.obj_dir cctx in
@@ -138,7 +138,7 @@ module Module = struct
                   let graph =
                     Dune_rules.Dep_graph.top_closed_implementations dep_graph [ module_ ]
                   in
-                  let+ modules, _ = Action_builder.run graph Eager in
+                  let+ modules, _ = Action_builder.evaluate_and_collect_facts graph in
                   modules
                 in
                 let cmos =
@@ -165,8 +165,8 @@ module Module = struct
       in
       let pps () =
         let module Merlin = Dune_rules.Merlin in
-        let pps = Merlin.pp_config merlin sctx ~expander in
-        let+ pps, _ = Action_builder.run pps Eager in
+        let pps = Merlin.pp_config merlin ctx ~expander in
+        let+ pps, _ = Action_builder.evaluate_and_collect_facts pps in
         let pp = Dune_rules.Module_name.Per_item.get pps module_name in
         match pp with
         | None -> None, None
@@ -187,11 +187,14 @@ module Module = struct
   ;;
 
   let term =
-    let+ common = Common.term
+    let+ builder = Common.Builder.term
     and+ module_path =
-      Arg.(required & pos 0 (some string) None & Arg.info [] ~docv:"MODULE")
+      Arg.(
+        required
+        & pos 0 (some string) None
+        & Arg.info [] ~docv:"MODULE" ~doc:"Path to an OCaml module.")
     and+ ctx_name = Common.context_arg ~doc:{|Select context where to build/run utop.|} in
-    let config = Common.init common in
+    let common, config = Common.init builder in
     Scheduler.go ~common ~config (fun () ->
       let open Fiber.O in
       let* setup = Import.Main.setup () in
@@ -203,12 +206,22 @@ module Module = struct
         in
         let+ directives =
           let module_path =
-            let root = Common.root common in
-            Path.Source.relative
-              Path.Source.root
-              (root.reach_from_root_prefix ^ module_path)
+            if Filename.is_relative module_path
+            then Path.Local.of_string module_path
+            else (
+              let root =
+                (Common.root common).dir
+                |> Path.of_string
+                |> Path.to_absolute_filename
+                |> Path.of_string
+              in
+              match Path.drop_prefix ~prefix:root (Path.of_string module_path) with
+              | Some module_path -> module_path
+              | None ->
+                User_error.raise
+                  [ Pp.text "Module path not a descendent of workspace root." ])
           in
-          module_directives sctx module_path
+          module_directives sctx (Path.Source.of_local module_path)
         in
         Dune_rules.Toplevel.print_toplevel_init_file directives))
   ;;
