@@ -102,12 +102,12 @@ module Status_line = struct
   let () = at_exit (fun () -> Printf.printf "\r%*s\r" (String.length !displayed) "")
 end
 
-(* Return list of entries in [path] as [path/entry] *)
+(* Return a sorted list of entries in [path] as [path/entry] *)
 let readdir path =
-  Array.fold_right
-    ~f:(fun entry dir -> (path ^/ entry) :: dir)
-    ~init:[]
-    (Sys.readdir path)
+  Sys.readdir path
+  |> Array.to_list
+  |> List.map ~f:(fun entry -> path ^/ entry)
+  |> List.sort ~cmp:String.compare
 ;;
 
 let open_out file =
@@ -445,7 +445,9 @@ end = struct
 
     let open_temp_file () =
       let out = Temp.file "duneboot-" ".output" in
-      let fd = Unix.openfile out [ O_WRONLY; O_CREAT; O_TRUNC; O_SHARE_DELETE ] 0o666 in
+      let fd =
+        Unix.openfile out [ O_WRONLY; O_CREAT; O_TRUNC; O_SHARE_DELETE; O_CLOEXEC ] 0o666
+      in
       out, fd
     ;;
 
@@ -691,8 +693,8 @@ module Build_info = struct
           "git"
           [ "describe"; "--always"; "--dirty"; "--abbrev=7" ]
         >>| (function
-        | Some s -> Some (String.trim s)
-        | None -> None)
+         | Some s -> Some (String.trim s)
+         | None -> None)
   ;;
 
   let gen_data_module oc =
@@ -1139,27 +1141,6 @@ let build
        ])
 ;;
 
-let build_with_single_command
-  ~ocaml_config:_
-  ~dependencies
-  ~c_files
-  ~link_flags
-  { target = name, main; external_libraries; _ }
-  =
-  let external_libraries, external_includes = resolve_externals external_libraries in
-  write_args "mods_list" (sort_files dependencies ~main);
-  Process.run
-    ~cwd:build_dir
-    Config.compiler
-    (List.concat
-       [ common_build_args name ~external_includes ~external_libraries
-       ; [ "-no-alias-deps"; "-w"; "-49-6" ]
-       ; c_files
-       ; [ "-args"; "mods_list" ] @ link_flags
-       ; allow_unstable_sources
-       ])
-;;
-
 let rec rm_rf fn =
   match Unix.lstat fn with
   | { st_kind = S_DIR; _ } ->
@@ -1191,7 +1172,6 @@ let main () =
        | None -> []
        | Some flags -> flags)
   in
-  let build = if concurrency = 1 || Sys.win32 then build_with_single_command else build in
   build ~ocaml_config ~dependencies ~c_files ~link_flags task
 ;;
 
