@@ -23,23 +23,28 @@ let local_libraries =
   ; ("otherlibs/dune-glob/src", Some "Dune_glob", false, None)
   ; ("otherlibs/xdg", Some "Xdg", false, None)
   ; ("otherlibs/dune-rpc/private", Some "Dune_rpc_private", false, None)
+  ; ("src/dune_config", Some "Dune_config", false, None)
+  ; ("vendor/sha", None, false, None)
+  ; ("vendor/uutf", None, false, None)
+  ; ("vendor/opam/src/core", None, false, None)
+  ; ("vendor/opam-file-format", None, false, None)
+  ; ("vendor/opam/src/format", None, false, None)
   ; ("otherlibs/chrome-trace/src", Some "Chrome_trace", false, None)
   ; ("vendor/spawn/src", Some "Spawn", false, None)
   ; ("src/dune_stats", Some "Dune_stats", false, None)
   ; ("vendor/build_path_prefix_map/src", Some "Build_path_prefix_map", false,
     None)
-  ; ("src/dune_config", Some "Dune_config", false, None)
   ; ("src/dune_util", Some "Dune_util", false, None)
+  ; ("src/dune_metrics", Some "Dune_metrics", false, None)
+  ; ("src/dune_digest", Some "Dune_digest", false, None)
   ; ("src/predicate_lang", Some "Predicate_lang", false, None)
   ; ("otherlibs/dune-private-libs/section", Some "Dune_section", false, None)
   ; ("src/dune_lang", Some "Dune_lang", false, None)
   ; ("src/fiber_event_bus", Some "Fiber_event_bus", false, None)
-  ; ("vendor/opam-file-format", None, false, None)
   ; ("src/dune_async_io", Some "Dune_async_io", false, None)
-  ; ("src/dune_metrics", Some "Dune_metrics", false, None)
-  ; ("src/dune_digest", Some "Dune_digest", false, None)
   ; ("src/fiber_util", Some "Fiber_util", false, None)
   ; ("src/dune_cache_storage", Some "Dune_cache_storage", false, None)
+  ; ("src/dune_targets", Some "Dune_targets", false, None)
   ; ("src/dune_cache", Some "Dune_cache", false, None)
   ; ("otherlibs/dune-action-plugin/src", Some "Dune_action_plugin", false,
     None)
@@ -61,21 +66,19 @@ let local_libraries =
     false, None)
   ; ("src/fs", Some "Fs", false, None)
   ; ("src/dune_findlib", Some "Dune_findlib", false, None)
-  ; ("vendor/sha", None, false, None)
-  ; ("vendor/uutf", None, false, None)
-  ; ("vendor/opam/src/core", None, false, None)
-  ; ("vendor/opam/src/format", None, false, None)
+  ; ("src/dune_vcs", Some "Dune_vcs", false, None)
   ; ("vendor/opam/src/repository", None, false, None)
   ; ("vendor/opam/src/state", None, false, None)
   ; ("vendor/0install-solver/src/solver", Some "Zeroinstall_solver", false,
     None)
   ; ("vendor/fmt/src", None, false, None)
   ; ("vendor/opam-0install/lib", Some "Opam_0install", false, None)
+  ; ("otherlibs/dune-build-info/src", Some "Build_info", false,
+    Some "Build_info_data")
   ; ("src/dune_pkg", Some "Dune_pkg", false, None)
   ; ("src/install", Some "Install", false, None)
   ; ("otherlibs/dune-site/src/private", Some "Dune_site_private", false,
     None)
-  ; ("src/dune_vcs", Some "Dune_vcs", false, None)
   ; ("src/dune_threaded_console", Some "Dune_threaded_console", false, None)
   ; ("vendor/lwd/lwd", None, false, None)
   ; ("vendor/notty/src", None, true, None)
@@ -88,9 +91,8 @@ let local_libraries =
   ; ("src/scheme", Some "Scheme", false, None)
   ; ("src/dune_rules", Some "Dune_rules", true, None)
   ; ("src/upgrader", Some "Dune_upgrader", false, None)
+  ; ("src/dune_pkg_outdated", Some "Dune_pkg_outdated", false, None)
   ; ("vendor/cmdliner/src", None, false, None)
-  ; ("otherlibs/dune-build-info/src", Some "Build_info", false,
-    Some "Build_info_data")
   ; ("src/dune_rpc_impl", Some "Dune_rpc_impl", false, None)
   ; ("src/dune_rules_rpc", Some "Dune_rules_rpc", false, None)
   ]
@@ -110,4 +112,5 @@ let link_flags =
     [ "-cclib"; "-lshell32"; "-cclib"; "-lole32"; "-cclib"; "-luuid" ])
   ; ("mingw64",
     [ "-cclib"; "-lshell32"; "-cclib"; "-lole32"; "-cclib"; "-luuid" ])
+  ; ("beos", [ "-cclib"; "-lbsd" ])
   ]
