@@ -3,8 +3,9 @@ let path_sep = if Sys.win32 then ';' else ':'
 let parse_path ?(sep = path_sep) s =
   String.split s ~on:sep
   |> List.filter_map ~f:(function
-       | "" -> None
-       | p -> Some (Path.of_filename_relative_to_initial_cwd p))
+    | "" -> None
+    | p -> Some (Path.of_filename_relative_to_initial_cwd p))
+;;
 
 let encode_strings paths = String.concat ~sep:(String.make 1 path_sep) paths
 
@@ -13,6 +14,7 @@ let cons_path ?(path_sep = path_sep) p ~_PATH =
   match _PATH with
   | None -> p
   | Some s -> Printf.sprintf "%s%c%s" p path_sep s
+;;
 
 let exe = if Sys.win32 then ".exe" else ""
 
@@ -21,13 +23,15 @@ let exists fn =
   | { st_kind = S_DIR; _ } -> false
   | exception Unix.Unix_error _ -> false
   | _ -> true
+;;
 
 let add_exe prog =
-  if String.is_suffix (String.lowercase prog) ~suffix:exe then prog
-  else prog ^ exe
+  if String.is_suffix (String.lowercase prog) ~suffix:exe then prog else prog ^ exe
+;;
 
 let which ~path prog =
   let prog = add_exe prog in
   List.find_map path ~f:(fun dir ->
-      let fn = Path.relative dir prog in
-      Option.some_if (exists fn) fn)
+    let fn = Path.relative dir prog in
+    Option.some_if (exists fn) fn)
+;;
