@@ -222,13 +222,15 @@ let exec_run_dynamic_client ~display ~ectx ~eenv prog args =
       match ectx.targets with
       | None -> String.Set.empty
       | Some targets ->
-        if not (Path.Build.Set.is_empty targets.dirs)
+        if not (Filename.Set.is_empty targets.dirs)
         then
           User_error.raise
             ~loc:ectx.rule_loc
             [ Pp.text "Directory targets are not compatible with dynamic actions" ];
-        Path.Build.Set.to_list_map targets.files ~f:(fun target ->
-          Path.reach (Path.build target) ~from:eenv.working_dir)
+        Filename.Set.to_list_map targets.files ~f:(fun target ->
+          Path.Build.relative targets.root target
+          |> Path.build
+          |> Path.reach ~from:eenv.working_dir)
         |> String.Set.of_list
     in
     { DAP.Run_arguments.prepared_dependencies = eenv.prepared_dependencies; targets }
@@ -322,8 +324,8 @@ let bash_exn =
 (* When passing these to an extension, they shouldn't need to know about any
    kind of dynamic build dependency functions or prepped dependencies, etc,
    which should be handled here instead. *)
-let restrict_ctx { targets; context; metadata; rule_loc; build_deps = _ } =
-  { Action.Ext.targets; context; purpose = metadata.purpose; rule_loc }
+let restrict_ctx { targets; context; metadata; rule_loc; build_deps } =
+  { Action.Ext.targets; context; purpose = metadata.purpose; rule_loc; build_deps }
 ;;
 
 let restrict_env
