@@ -1,11 +1,12 @@
-open Import
-
 (** Action runners are instances capabale of executing dune actions outside of
     the build engine's process. *)
 
+type t
+
 module Rpc_server : sig
   (** The component of the RPC server required to orchestrate the runners. It's
       responsible for handing off sessions to action runners once they connect. *)
+  type runner := t
 
   type t
 
@@ -20,19 +21,21 @@ module Rpc_server : sig
 
   (** [stop t] is to be run by the rpc server *)
   val stop : t -> unit Fiber.t
+
+  val all_runners : t -> runner list
 end
 
-type t
-
 val create : Rpc_server.t -> name:string -> t
-
 val name : t -> string
 
+(* CR-soon dkalinichenko: return [Exn_with_backtrace.t list] in the error case
+   after rgrinberg patches exception marshalling upstream. *)
+
 (** [exec_action worker action] dispatches [action] to [worker] *)
-val exec_action :
-     t
-  -> Action_exec.input
-  -> (Action_exec.Exec_result.t, Exn_with_backtrace.t list) result Fiber.t
+val exec_action : t -> Action_exec.input -> Action_exec.Exec_result.t Fiber.t
+
+(** [cancel_build] cancels all actions being executed by [worker] *)
+val cancel_build : t -> unit Fiber.t
 
 module Worker : sig
   (** A worker is a runner of action *)
