@@ -8,18 +8,11 @@ module Name = struct
   let of_string s =
     match of_string_opt s with
     | Some s -> s
-    | None -> Code_error.raise "invalid alias name" [ ("s", Dyn.string s) ]
+    | None -> Code_error.raise "invalid alias name" [ "s", Dyn.string s ]
+  ;;
 
   let default = of_string "default"
 
-  let runtest = of_string "runtest"
-
-  let install = of_string "install"
-
-  let fmt = of_string "fmt"
-
-  let all = of_string "all"
-
   let parse_local_path (loc, p) =
     match Path.Local.parent p with
     | Some dir ->
@@ -27,10 +20,10 @@ module Name = struct
       , (* TODO one day we should validate the name properly here *)
         Path.Local.basename p |> of_string_opt_loose |> Option.value_exn )
     | None ->
-      User_error.raise ~loc
-        [ Pp.textf "Invalid alias path: %S"
-            (Path.Local.to_string_maybe_quoted p)
-        ]
+      User_error.raise
+        ~loc
+        [ Pp.textf "Invalid alias path: %S" (Path.Local.to_string_maybe_quoted p) ]
+  ;;
 end
 
 module T : sig
@@ -40,7 +33,6 @@ module T : sig
     }
 
   val make : Name.t -> dir:Path.Build.t -> t
-
   val of_user_written_path : loc:Loc.t -> Path.t -> t
 end = struct
   type t =
@@ -58,11 +50,14 @@ end = struct
       in
       { dir = Path.Build.parent_exn path; name }
     | None ->
-      User_error.raise ~loc
+      User_error.raise
+        ~loc
         [ Pp.text "Invalid alias!"
-        ; Pp.textf "Tried to reference path outside build dir: %S"
+        ; Pp.textf
+            "Tried to reference path outside build dir: %S"
             (Path.to_string_maybe_quoted path)
         ]
+  ;;
 end
 
 include T
@@ -71,81 +66,66 @@ let compare { dir; name } t =
   let open Ordering.O in
   let= () = Name.compare name t.name in
   Path.Build.compare dir t.dir
+;;
 
 let equal x y = compare x y = Eq
-
 let hash { dir; name } = Tuple.T2.hash Path.Build.hash Name.hash (dir, name)
 
 let to_dyn { dir; name } =
   let open Dyn in
-  Record [ ("dir", Path.Build.to_dyn dir); ("name", Name.to_dyn name) ]
+  Record [ "dir", Path.Build.to_dyn dir; "name", Name.to_dyn name ]
+;;
 
 let name t = t.name
-
 let dir t = t.dir
-
 let fully_qualified_name t = Path.Build.relative t.dir (Name.to_string t.name)
 
 (* This mutable table is safe: it's modified only at the top level. *)
 let standard_aliases = Table.create (module Name) 7
-
 let is_standard name = Table.mem standard_aliases name
 
 let make_standard name =
   Table.add_exn standard_aliases name ();
   make name
+;;
 
 let register_as_standard name =
   let (_ : (unit, _) result) = Table.add standard_aliases name () in
   ()
+;;
 
 let default = make_standard Name.default
 
-let runtest = make_standard Name.runtest
-
-let install = make_standard Name.install
-
-let doc = make_standard (Name.of_string "doc")
-
-let private_doc = make_standard (Name.of_string "doc-private")
-
-let lint = make_standard (Name.of_string "lint")
-
-let all = make_standard (Name.of_string "all")
-
-let check = make_standard (Name.of_string "check")
-
-let fmt = make_standard Name.fmt
-
 let encode { dir; name } =
   let open Dune_sexp.Encoder in
-  record [ ("dir", Dpath.encode (Path.build dir)); ("name", Name.encode name) ]
+  record [ "dir", Dpath.encode (Path.build dir); "name", Name.encode name ]
+;;
 
 let get_ctx (path : Path.Build.t) =
   match Path.Build.extract_first_component path with
   | None -> None
-  | Some (name, sub) -> (
-    match Context_name.of_string_opt name with
-    | None -> None
-    | Some ctx -> Some (ctx, Path.Source.of_local sub))
+  | Some (name, sub) ->
+    (match Context_name.of_string_opt name with
+     | None -> None
+     | Some ctx -> Some (ctx, Path.Source.of_local sub))
+;;
 
 let describe ?(loc = Loc.none) alias =
   let open Pp.O in
   let pp =
     match get_ctx alias.dir with
     | None ->
-      Pp.textf "invalid-alias %s"
-        (Path.Build.to_string (fully_qualified_name alias))
+      Pp.textf "invalid-alias %s" (Path.Build.to_string (fully_qualified_name alias))
     | Some (ctx, dir_in_context) ->
       let pp =
         Pp.textf "alias "
         ++ Pp.verbatim
              (Path.Source.to_string_maybe_quoted
-                (Path.Source.relative dir_in_context
-                   (Name.to_string alias.name)))
+                (Path.Source.relative dir_in_context (Name.to_string alias.name)))
       in
-      if Context_name.is_default ctx then pp
+      if Context_name.is_default ctx
+      then pp
       else pp ++ Pp.textf " (context %s)" (Context_name.to_string ctx)
   in
-  if Loc.is_none loc then pp
-  else pp ++ Pp.textf " in %s" (Loc.to_file_colon_line loc)
+  if Loc.is_none loc then pp else pp ++ Pp.textf " in %s" (Loc.to_file_colon_line loc)
+;;
