@@ -13,12 +13,12 @@ module T = struct
   module Stable_for_digest = struct
     type t =
       | Env of string
-      | File of string
+      | File of Digest.t (* Digest of the underlying [Path.t] *)
       | Alias of
           { dir : string
           ; name : string
           }
-      | File_selector of Dyn.t
+      | File_selector of Digest.t (* Digest of the underlying [File_selector.t] *)
       | Universe
   end
 
@@ -76,53 +76,80 @@ let as_in_build_dir_no_source = function
 ;;
 
 module Fact = struct
-  (* CR-someday amokhov: Find a better name, perhaps, [Files_and_dirs]? *)
   module Files = struct
     type t =
-      { files : Digest.t Path.Map.t
-      ; dirs : Digest.t Path.Map.t (* Only for file selectors for now *)
-      ; digest : Digest.t
+      { files : Path.Set.t
+      ; empty_dirs : Path.Build.Set.t (* For [File_selector]s that match no files *)
+      ; digest : Digest.t (* Includes [empty_dirs], [files] and their content digests *)
       }
 
-    let to_dyn { files; dirs; digest } =
+    let to_dyn { files; empty_dirs; digest } =
       Dyn.Record
-        [ "files", Path.Map.to_dyn Digest.to_dyn files
-        ; "dirs", Path.Map.to_dyn Digest.to_dyn dirs
+        [ "files", Path.Set.to_dyn files
+        ; "empty_dirs", Path.Build.Set.to_dyn empty_dirs
         ; "digest", Digest.to_dyn digest
         ]
     ;;
 
-    let is_empty t = Path.Map.is_empty t.files && Path.Map.is_empty t.dirs
+    let is_empty t = Path.Set.is_empty t.files && Path.Build.Set.is_empty t.empty_dirs
     let compare a b = Digest.compare a.digest b.digest
     let equal a b = Digest.equal a.digest b.digest
-    let paths t = t.files
 
-    let make ~files ~dirs =
-      { files
-      ; dirs
-      ; digest =
-          Digest.generic
-            ( Path.Map.to_list_map files ~f:(fun p d -> Path.to_string p, d)
-            , Path.Map.to_list_map dirs ~f:(fun p d -> Path.to_string p, d) )
+    (* The caller should ensure that [files] and [digests] are listed in the same order *)
+    let combined_digest ?empty_dir (files : Path.t list) (digests : Digest.t list) =
+      let files = List.map files ~f:Path.to_string in
+      match empty_dir with
+      | None -> Digest.generic (files, digests)
+      | Some empty_dir -> Digest.generic (empty_dir, files, digests)
+    ;;
+
+    let create files ~build_file =
+      let open Memo.O in
+      let empty_dir =
+        if Filename_set.is_empty files
+        then Path.as_in_build_dir (Filename_set.dir files)
+        else None
+      in
+      let empty_dirs =
+        match empty_dir with
+        | None -> Path.Build.Set.empty
+        | Some dir -> Path.Build.Set.singleton dir
+      in
+      let files = Filename_set.to_list files in
+      let+ digests = Memo.parallel_map files ~f:build_file in
+      { files = Path.Set.of_list files
+      ; empty_dirs
+      ; digest = combined_digest ?empty_dir files digests
       }
     ;;
 
-    let necessary_dirs_for_sandboxing { files; dirs; digest = _ } =
-      let f (path : Path.t) (_ : Digest.t) acc =
+    let of_file_digest_map file_digest_map =
+      let files = Path.Set.of_keys file_digest_map in
+      let digests = Path.Map.values file_digest_map in
+      { files
+      ; empty_dirs = Path.Build.Set.empty
+      ; digest = combined_digest (Path.Set.to_list files) digests
+      }
+    ;;
+
+    let necessary_dirs_for_sandboxing { files; empty_dirs; digest = _ } =
+      let f (path : Path.t) acc =
         match as_in_build_dir_no_source path with
         | None -> acc
         | Some p -> Path.Build.Set.add acc (Path.Build.parent_exn p)
       in
-      let init = Path.Map.foldi files ~init:Path.Build.Set.empty ~f in
-      Path.Map.foldi dirs ~init ~f
+      Path.Set.fold files ~init:empty_dirs ~f
     ;;
 
-    let empty = lazy (make ~files:Path.Map.empty ~dirs:Path.Map.empty)
+    let empty =
+      { files = Path.Set.empty
+      ; empty_dirs = Path.Build.Set.empty
+      ; digest = Digest.generic []
+      }
+    ;;
 
     let group ts files =
-      let ts =
-        if Path.Map.is_empty files then ts else make ~files ~dirs:Path.Map.empty :: ts
-      in
+      let ts = if Path.Map.is_empty files then ts else of_file_digest_map files :: ts in
       (* Sort and de-dup so that the result is resilient to code changes *)
       let ts =
         List.filter_map ts ~f:(fun t -> if is_empty t then None else Some (t.digest, t))
@@ -130,19 +157,11 @@ module Fact = struct
         |> Digest.Map.values
       in
       match ts with
-      | [] -> Lazy.force empty
+      | [] -> empty
       | [ t ] -> t
-      | t :: l ->
-        { files =
-            List.fold_left l ~init:t.files ~f:(fun acc t ->
-              Path.Map.union t.files acc ~f:(fun _ d1 d2 ->
-                assert (Digest.equal d1 d2);
-                Some d1))
-        ; dirs =
-            List.fold_left l ~init:t.dirs ~f:(fun acc t ->
-              Path.Map.union t.dirs acc ~f:(fun _ d1 d2 ->
-                assert (Digest.equal d1 d2);
-                Some d1))
+      | ts ->
+        { files = Path.Set.union_map ts ~f:(fun t -> t.files)
+        ; empty_dirs = Path.Build.Set.union_map ts ~f:(fun t -> t.empty_dirs)
         ; digest = Digest.generic (List.map ts ~f:(fun t -> t.digest))
         }
     ;;
@@ -151,7 +170,10 @@ module Fact = struct
   type t =
     | Nothing
     | File of Path.t * Digest.t
-    | File_selector of Dyn.t * Files.t
+    | File_selector of
+        { file_selector_digest : Digest.t
+        ; facts : Files.t
+        }
     | Alias of Files.t
 
   let to_dyn = function
@@ -160,17 +182,28 @@ module Fact = struct
       Dyn.Variant
         ( "File"
         , [ Dyn.Record [ "path", Path.to_dyn path; "digest", Digest.to_dyn digest ] ] )
-    | File_selector (dyn, files) ->
+    | File_selector { file_selector_digest; facts } ->
       Dyn.Variant
-        ("File_selector", [ Dyn.Record [ "dyn", dyn; "files", Files.to_dyn files ] ])
-    | Alias files -> Dyn.Variant ("Alias", [ Dyn.Record [ "files", Files.to_dyn files ] ])
+        ( "File_selector"
+        , [ Dyn.Record
+              [ "file_selector_digest", Digest.to_dyn file_selector_digest
+              ; "facts", Files.to_dyn facts
+              ]
+          ] )
+    | Alias facts -> Dyn.Variant ("Alias", [ Dyn.Record [ "facts", Files.to_dyn facts ] ])
   ;;
 
   module Stable_for_digest = struct
     type t =
       | Env of string * string option
-      | File of string * Digest.t
-      | File_selector of Dyn.t * Digest.t
+      | File of
+          { path_digest : Digest.t
+          ; file_digest : Digest.t
+          }
+      | File_selector of
+          { file_selector_digest : Digest.t
+          ; facts_digest : Digest.t
+          }
       | Alias of Digest.t
   end
 
@@ -185,8 +218,9 @@ module Fact = struct
       Digest.compare d1 d2
     | File _, _ -> Lt
     | _, File _ -> Gt
-    | File_selector (d1, f1), File_selector (d2, f2) ->
-      let= () = Dyn.compare d1 d2 in
+    | ( File_selector { file_selector_digest = d1; facts = f1 }
+      , File_selector { file_selector_digest = d2; facts = f2 } ) ->
+      let= () = Digest.compare d1 d2 in
       Files.compare f1 f2
     | File_selector _, _ -> Lt
     | _, File_selector _ -> Gt
@@ -202,94 +236,17 @@ module Fact = struct
   let nothing = Nothing
   let file fn digest = File (fn, digest)
 
-  let file_selector fs files =
-    let id = File_selector.to_dyn fs in
-    File_selector (id, files)
+  let file_selector fs facts =
+    (* CR-someday amokhov: We used to call [File_selector.to_dyn] here that raises under
+       the same conditions that [File_selector.digest_exn] is raising, namely, when
+       the underlying glob is not serialisable. We should make all globs serialisable
+       or use stronger types to statically rule out the possibility of raising here. *)
+    File_selector { file_selector_digest = File_selector.digest_exn fs; facts }
   ;;
 
   let alias _alias files = Alias files
 end
 
-module Facts = struct
-  type t = Fact.t Map.t
-
-  let empty = Map.empty
-
-  let union a b =
-    Map.union a b ~f:(fun _ a b ->
-      assert (a = b);
-      Some a)
-  ;;
-
-  let union_all xs = List.fold_left xs ~init:Map.empty ~f:union
-  let to_dyn = Map.to_dyn Fact.to_dyn
-
-  let paths t =
-    Map.fold t ~init:Path.Map.empty ~f:(fun fact acc ->
-      match (fact : Fact.t) with
-      | Nothing -> acc
-      | File (p, d) -> Path.Map.set acc p d
-      | File_selector (_, ps) | Alias ps ->
-        Path.Map.union acc ps.files ~f:(fun _ a _ -> Some a))
-  ;;
-
-  let paths_without_expanding_aliases t =
-    Map.fold t ~init:Path.Map.empty ~f:(fun fact acc ->
-      match (fact : Fact.t) with
-      | Nothing | Alias _ -> acc
-      | File (p, d) -> Path.Map.set acc p d
-      | File_selector (_, ps) -> Path.Map.union acc ps.files ~f:(fun _ a _ -> Some a))
-  ;;
-
-  let group_paths_as_fact_files ts =
-    let fact_files, paths =
-      List.fold_left ts ~init:([], Path.Map.empty) ~f:(fun acc t ->
-        Map.fold t ~init:acc ~f:(fun fact ((acc_ff, acc_paths) as acc) ->
-          match (fact : Fact.t) with
-          | Nothing -> acc
-          | File (p, d) -> acc_ff, Path.Map.set acc_paths p d
-          | File_selector (_, ps) | Alias ps -> ps :: acc_ff, acc_paths))
-    in
-    Fact.Files.group fact_files paths
-  ;;
-
-  let necessary_dirs_for_sandboxing t =
-    Map.fold t ~init:Path.Build.Set.empty ~f:(fun fact acc ->
-      match (fact : Fact.t) with
-      | Nothing -> acc
-      | File (p, _) ->
-        (match as_in_build_dir_no_source p with
-         | None -> acc
-         | Some p ->
-           let p = Path.Build.parent_exn p in
-           Path.Build.Set.add acc p)
-      | File_selector (_, ps) | Alias ps ->
-        Path.Build.Set.union_all
-          [ acc
-          ; Path.Map.keys ps.dirs
-            |> List.filter_map ~f:as_in_build_dir_no_source
-            |> Path.Build.Set.of_list
-          ; Fact.Files.necessary_dirs_for_sandboxing ps
-          ])
-  ;;
-
-  let digest t ~env =
-    let facts =
-      Map.foldi t ~init:[] ~f:(fun dep fact acc : Fact.Stable_for_digest.t list ->
-        match dep with
-        | Env var -> Env (var, Env.get env var) :: acc
-        | Universe -> acc
-        | File _ | File_selector _ | Alias _ ->
-          (match (fact : Fact.t) with
-           | Nothing -> acc
-           | File (p, d) -> File (Path.to_string p, d) :: acc
-           | File_selector (id, ps) -> File_selector (id, ps.digest) :: acc
-           | Alias ps -> Alias ps.digest :: acc))
-    in
-    Digest.generic facts
-  ;;
-end
-
 module Set = struct
   module M = Set.Of_map (T) (Map)
   include M
@@ -317,8 +274,13 @@ module Set = struct
       match dep with
       | Env var -> Env var :: acc
       | Universe -> Universe :: acc
-      | File p -> File (Path.to_string p) :: acc
-      | File_selector fs -> File_selector (File_selector.to_dyn fs) :: acc
+      | File p -> File (Path.to_string p |> Digest.string) :: acc
+      | File_selector fs ->
+        (* CR-someday amokhov: We used to call [File_selector.to_dyn] here that raises under
+           the same conditions that [File_selector.digest_exn] is raising, namely, when
+           the underlying glob is not serialisable. We should make all globs serialisable
+           or use stronger types to statically rule out the possibility of raising here. *)
+        File_selector (File_selector.digest_exn fs) :: acc
       | Alias a ->
         Alias
           { dir = Path.Build.to_string (Alias.dir a)
@@ -328,3 +290,77 @@ module Set = struct
     |> Digest.generic
   ;;
 end
+
+module Facts = struct
+  type t = Fact.t Map.t
+
+  let singleton = Map.singleton
+  let equal x y = Map.equal ~equal:Fact.equal x y
+  let empty = Map.empty
+  let record_facts set ~f = Map.parallel_map set ~f:(fun dep () -> f dep)
+
+  let union a b =
+    Map.union a b ~f:(fun _ a b ->
+      assert (a = b);
+      Some a)
+  ;;
+
+  let union_all xs = List.fold_left xs ~init:Map.empty ~f:union
+  let to_dyn = Map.to_dyn Fact.to_dyn
+
+  let paths t ~expand_aliases =
+    Map.fold t ~init:Path.Set.empty ~f:(fun fact acc ->
+      match (fact : Fact.t) with
+      | Nothing -> acc
+      | File (path, _digest) -> Path.Set.add acc path
+      | File_selector { file_selector_digest = _; facts } ->
+        Path.Set.union acc facts.files
+      | Alias facts -> if expand_aliases then Path.Set.union acc facts.files else acc)
+  ;;
+
+  let group_paths_as_fact_files ts =
+    let fact_files, paths =
+      List.fold_left ts ~init:([], Path.Map.empty) ~f:(fun acc t ->
+        Map.fold t ~init:acc ~f:(fun fact ((acc_ff, acc_paths) as acc) ->
+          match (fact : Fact.t) with
+          | Nothing -> acc
+          | File (p, d) -> acc_ff, Path.Map.set acc_paths p d
+          | File_selector { file_selector_digest = _; facts } | Alias facts ->
+            facts :: acc_ff, acc_paths))
+    in
+    Fact.Files.group fact_files paths
+  ;;
+
+  let necessary_dirs_for_sandboxing t =
+    Map.fold t ~init:Path.Build.Set.empty ~f:(fun fact acc ->
+      match (fact : Fact.t) with
+      | Nothing -> acc
+      | File (p, _) ->
+        (match as_in_build_dir_no_source p with
+         | None -> acc
+         | Some p ->
+           let p = Path.Build.parent_exn p in
+           Path.Build.Set.add acc p)
+      | File_selector { file_selector_digest = _; facts } | Alias facts ->
+        Path.Build.Set.union_all [ acc; Fact.Files.necessary_dirs_for_sandboxing facts ])
+  ;;
+
+  let digest t ~env =
+    let facts =
+      Map.foldi t ~init:[] ~f:(fun dep fact acc : Fact.Stable_for_digest.t list ->
+        match dep with
+        | Env var -> Env (var, Env.get env var) :: acc
+        | Universe -> acc
+        | File _ | File_selector _ | Alias _ ->
+          (match (fact : Fact.t) with
+           | Nothing -> acc
+           | File (p, d) ->
+             File { path_digest = Digest.string (Path.to_string p); file_digest = d }
+             :: acc
+           | File_selector { file_selector_digest; facts } ->
+             File_selector { file_selector_digest; facts_digest = facts.digest } :: acc
+           | Alias ps -> Alias ps.digest :: acc))
+    in
+    Digest.generic facts
+  ;;
+end
