@@ -8,7 +8,6 @@ module Target_dir : sig
   val build_dir : context_related -> Path.Build.t
 
   type t =
-    | Install of context_related
     | Anonymous_action of context_related
     | Regular of context_related
     | Invalid of Path.Build.t
@@ -20,7 +19,6 @@ type target_kind =
   | Regular of Context_name.t * Path.Source.t
   | Alias of Context_name.t * Path.Source.t
   | Anonymous_action of Context_name.t
-  | Install of Context_name.t * Path.Source.t
   | Other of Path.Build.t
 
 type 'build path_kind =
@@ -32,7 +30,6 @@ type 'build path_kind =
 val analyse_target : Path.Build.t -> target_kind
 
 val analyse_path : Path.t -> target_kind path_kind
-
 val analyse_dir : Path.t -> Target_dir.t path_kind
 
 (** Nice description of a target *)
@@ -40,26 +37,13 @@ val describe_target : Path.Build.t -> string
 
 val describe_path : Path.t -> string
 
-include Dune_sexp.Conv.S with type t = Path.t
+type t = Path.t
 
-module Local : sig
-  val encode : dir:Path.t -> Path.t Dune_sexp.Encoder.t
-
-  val decode : dir:Path.t -> Path.t Dune_sexp.Decoder.t
-end
+val encode : Path.t Dune_sexp.Encoder.t
 
 module Build : sig
-  include Dune_sexp.Conv.S with type t = Path.Build.t
-
-  val is_dev_null : t -> bool
-
-  val install_dir : t
+  type t = Path.Build.t
 
   val anonymous_actions_dir : t
-end
-
-module External : sig
-  val encode : Path.External.t Dune_sexp.Encoder.t
-
-  val decode : Path.External.t Dune_sexp.Decoder.t
+  val anonymous_actions_dir_basename : Filename.t
 end
