@@ -13,7 +13,7 @@ module Alias = Alias
 module Dpath = Dpath
 module Rules = Rules
 module Rule = Rule
-module Targets = Targets
+module Targets = Dune_targets
 module Target_promotion = Target_promotion
 module Build_context = Build_context
 module Build_config = Build_config
