@@ -1,20 +1,15 @@
-open Stdune
+open Import
 
 module type S = sig
   val always : (unit -> unit) -> unit
-
   val once : (unit -> unit) -> unit
-
   val run : unit -> unit
 end
 
 module Make () = struct
   let persistent_hooks = ref []
-
   let one_off_hooks = ref []
-
   let always hook = persistent_hooks := hook :: !persistent_hooks
-
   let once hook = one_off_hooks := hook :: !one_off_hooks
 
   let run () =
@@ -32,9 +27,9 @@ module Make () = struct
     | [] -> ()
     | exns ->
       let exns = List.rev exns in
-      let open Dyn.Encoder in
-      Code_error.raise "hooks failed"
-        [ ("exns", (list Exn_with_backtrace.to_dyn) exns) ]
+      let open Dyn in
+      Code_error.raise "hooks failed" [ "exns", (list Exn_with_backtrace.to_dyn) exns ]
+  ;;
 end
 
 module End_of_build = Make ()
