@@ -8,12 +8,13 @@ let resolve_link_for_git path =
   | Error Not_a_symlink -> path
   | Error Max_depth_exceeded ->
     User_error.raise
-      [ Pp.textf "Unable to resolve symlink %s. Max recursion depth exceeded"
+      [ Pp.textf
+          "Unable to resolve symlink %s. Max recursion depth exceeded"
           (Path.to_string path)
       ]
   | Error (Unix_error _) ->
-    User_error.raise
-      [ Pp.textf "Unable to resolve symlink %s" (Path.to_string path) ]
+    User_error.raise [ Pp.textf "Unable to resolve symlink %s" (Path.to_string path) ]
+;;
 
 module Diff = struct
   type t =
@@ -23,8 +24,9 @@ module Diff = struct
 
   let print { loc; output } =
     Option.iter loc ~f:(fun loc ->
-        Loc.pp loc |> Pp.map_tags ~f:(fun Loc.Loc -> []) |> Ansi_color.print);
+      Loc.pp loc |> Pp.map_tags ~f:(fun Loc.Loc -> []) |> Ansi_color.print);
     print_string output
+  ;;
 end
 
 type command =
@@ -38,11 +40,8 @@ module With_fallback : sig
   type t
 
   val fail : User_message.t -> t
-
   val run : command -> fallback:t -> t
-
   val exec : t -> _ Fiber.t
-
   val capture : t -> (Diff.t, User_message.t) result Fiber.t
 end = struct
   type t =
@@ -52,6 +51,7 @@ end = struct
 
   let run command ~fallback:{ commands; error } =
     { commands = command :: commands; error }
+  ;;
 
   let fail error = { commands = []; error }
 
@@ -59,21 +59,28 @@ end = struct
     | { commands = []; error } -> raise (User_error.E error)
     | { commands = { dir; metadata; prog; args } :: commands; error } ->
       let* () =
-        Process.run ~display:Quiet ~dir ~env:Env.initial Strict prog args
-          ~metadata
+        Process.run ~display:Quiet ~dir ~env:Env.initial Strict prog args ~metadata
       in
       exec { commands; error }
+  ;;
 
   let rec capture = function
     | { commands = []; error } -> Fiber.return (Error error)
-    | { commands = { dir; metadata; prog; args } :: commands; error } -> (
+    | { commands = { dir; metadata; prog; args } :: commands; error } ->
       let* output, code =
-        Process.run_capture ~display:Quiet ~dir ~env:Env.initial Return prog
-          args ~metadata
+        Process.run_capture
+          ~display:Quiet
+          ~dir
+          ~env:Env.initial
+          Return
+          prog
+          args
+          ~metadata
       in
-      match code with
-      | 1 -> Fiber.return (Ok { Diff.output; loc = metadata.loc })
-      | _ -> capture { commands; error })
+      (match code with
+       | 1 -> Fiber.return (Ok { Diff.output; loc = metadata.loc })
+       | _ -> capture { commands; error })
+  ;;
 end
 
 let prepare ~skip_trailing_cr annots path1 path2 =
@@ -83,25 +90,29 @@ let prepare ~skip_trailing_cr annots path1 path2 =
       , Path.extract_build_context_dir_maybe_sandboxed path2 )
     with
     | Some (dir1, f1), Some (dir2, f2) when Path.equal dir1 dir2 ->
-      (dir1, Path.source f1, Path.source f2)
-    | _ -> (Path.root, path1, path2)
+      dir1, Path.source f1, Path.source f2
+    | _ -> Path.root, path1, path2
   in
   let loc = Loc.in_file file1 in
-  let run ?(dir = dir)
-      ?(metadata =
-        Process.create_metadata ~purpose:Internal_job ~loc ~annots ()) prog args
-      ~fallback =
+  let run
+    ?(dir = dir)
+    ?(metadata = Process.create_metadata ~purpose:Internal_job ~loc ~annots ())
+    prog
+    args
+    ~fallback
+    =
     With_fallback.run { dir; prog; args; metadata } ~fallback
   in
   let file1, file2 = Path.(to_string file1, to_string file2) in
   let fallback =
     With_fallback.fail
-      (User_error.make ~loc ~annots
-         [ Pp.textf "Files %s and %s differ."
-             (Path.to_string_maybe_quoted
-                (Path.drop_optional_sandbox_root path1))
-             (Path.to_string_maybe_quoted
-                (Path.drop_optional_sandbox_root path2))
+      (User_error.make
+         ~loc
+         ~annots
+         [ Pp.textf
+             "Files %s and %s differ."
+             (Path.to_string_maybe_quoted (Path.drop_optional_sandbox_root path1))
+             (Path.to_string_maybe_quoted (Path.drop_optional_sandbox_root path2))
          ])
   in
   let normal_diff () =
@@ -124,18 +135,15 @@ let prepare ~skip_trailing_cr annots path1 path2 =
           , List.map
               ~f:(fun path -> resolve_link_for_git path |> Path.reach ~from:dir)
               [ path1; path2 ] )
-      | None -> (
-        match which "diff" with
-        | Some path ->
-          Some (dir, path, [ "-u" ], "--strip-trailing-cr", [ file1; file2 ])
-        | None -> None)
+      | None ->
+        (match which "diff" with
+         | Some path -> Some (dir, path, [ "-u" ], "--strip-trailing-cr", [ file1; file2 ])
+         | None -> None)
     in
     match diff with
     | None -> fallback
     | Some (dir, path, args, skip_trailing_cr_arg, files) ->
-      let args =
-        if skip_trailing_cr then args @ [ skip_trailing_cr_arg ] else args
-      in
+      let args = if skip_trailing_cr then args @ [ skip_trailing_cr_arg ] else args in
       let args = args @ files in
       run ~dir path args ~fallback
   in
@@ -144,32 +152,38 @@ let prepare ~skip_trailing_cr annots path1 path2 =
   | Some cmd ->
     let sh, arg = Utils.system_shell_exn ~needed_to:"print diffs" in
     let cmd =
-      sprintf "%s %s %s" cmd
-        (String.quote_for_shell file1)
-        (String.quote_for_shell file2)
+      sprintf "%s %s %s" cmd (String.quote_for_shell file1) (String.quote_for_shell file2)
     in
-    run sh [ arg; cmd ]
+    run
+      sh
+      [ arg; cmd ]
       ~fallback:
         (With_fallback.fail
-           (User_error.make ~loc ~annots
-              [ Pp.textf "command reported no differences: %s"
-                  (if Path.is_root dir then cmd
+           (User_error.make
+              ~loc
+              ~annots
+              [ Pp.textf
+                  "command reported no differences: %s"
+                  (if Path.is_root dir
+                   then cmd
                    else
-                     sprintf "cd %s && %s"
+                     sprintf
+                       "cd %s && %s"
                        (String.quote_for_shell (Path.to_string dir))
                        cmd)
               ]))
-  | None -> (
-    if Execution_env.inside_dune then fallback
-    else
+  | None ->
+    if Execution_env.inside_dune
+    then fallback
+    else (
       match Bin.which ~path:(Env_path.path Env.initial) "patdiff" with
       | None -> normal_diff ()
       | Some prog ->
-        run prog
+        run
+          prog
           ([ "-keep-whitespace"; "-location-style"; "omake" ]
-          @ (if Lazy.force Ansi_color.stderr_supports_color then []
-             else [ "-ascii" ])
-          @ [ file1; file2 ])
+           @ (if Lazy.force Ansi_color.stderr_supports_color then [] else [ "-ascii" ])
+           @ [ file1; file2 ])
           ~metadata:
             ((* Because of the [-location-style omake], patdiff will print the
                 location of each hunk in a format that the editor should
@@ -178,19 +192,25 @@ let prepare ~skip_trailing_cr annots path1 path2 =
                 output has a location.
 
                 For this reason, we manually pass the below annotation. *)
-             Process.create_metadata ~purpose:Internal_job ~loc
+             Process.create_metadata
+               ~purpose:Internal_job
+               ~loc
                ~annots:
-                 (User_message.Annots.set annots
-                    User_message.Annots.has_embedded_location ())
+                 (User_message.Annots.set
+                    annots
+                    User_message.Annots.has_embedded_location
+                    ())
                ())
-          ~fallback:
-            ((* Use "diff" if "patdiff" reported no differences *)
-             normal_diff ()))
+          ~fallback:((* Use "diff" if "patdiff" reported no differences *)
+                     normal_diff ()))
+;;
 
 let print ?(skip_trailing_cr = Sys.win32) annots path1 path2 =
   let p = prepare ~skip_trailing_cr annots path1 path2 in
   With_fallback.exec p
+;;
 
 let get ?(skip_trailing_cr = Sys.win32) annots path1 path2 =
   let p = prepare ~skip_trailing_cr annots path1 path2 in
   With_fallback.capture p
+;;
