@@ -1,5 +1,4 @@
 open Import
-module Action_builder = Action_builder0
 module Id = Id.Make ()
 
 module Dir_rules = struct
@@ -10,7 +9,6 @@ module Dir_rules = struct
 
     type t = { expansions : (Loc.t * item) Appendable_list.t } [@@unboxed]
 
-    let empty = { expansions = Appendable_list.empty }
     let union x y = { expansions = Appendable_list.( @ ) x.expansions y.expansions }
   end
 
@@ -40,19 +38,23 @@ module Dir_rules = struct
     }
 
   let consume t =
-    let data = Id.Map.values t in
-    let rules =
-      List.filter_map data ~f:(function
-        | Rule rule -> Some rule
-        | Alias _ -> None)
+    let rules, aliases =
+      Id.Map.values t
+      |> List.partition_map ~f:(function
+        | Rule rule -> Left rule
+        | Alias { name; spec } -> Right (name, spec))
     in
     let aliases =
-      Alias.Name.Map.of_list_multi
-        (List.filter_map data ~f:(function
-          | Rule _ -> None
-          | Alias { name; spec } -> Some (name, spec)))
-      |> Alias.Name.Map.map ~f:(fun specs ->
-        List.fold_left specs ~init:Alias_spec.empty ~f:Alias_spec.union)
+      let add_item what = function
+        | None -> Some what
+        | Some base -> Some (Alias_spec.union what base)
+      in
+      (* This accumulates the aliases in reverse order, but there's another
+         reversal whenever the expansion is inspected. The order doesn't really
+         matter, but it does change the tests. So it's nice to maintain it if
+         possible *)
+      List.fold_left aliases ~init:Alias.Name.Map.empty ~f:(fun acc (name, item) ->
+        Alias.Name.Map.update acc name ~f:(add_item item))
     in
     { rules; aliases }
   ;;
@@ -114,7 +116,7 @@ include T
 let to_dyn = Path.Build.Map.to_dyn Dir_rules.Nonempty.to_dyn
 
 let singleton_rule (rule : Rule.t) =
-  let dir = rule.dir in
+  let dir = rule.targets.root in
   Path.Build.Map.singleton dir (Dir_rules.Nonempty.singleton (Rule rule))
 ;;
 
@@ -148,12 +150,11 @@ module Produce = struct
         }
     ;;
 
-    let add_action t ~context ~loc action =
+    let add_action t ~loc action =
       let action =
         let open Action_builder.O in
         let+ action = action in
-        { Rule.Anonymous_action.context = Some context
-        ; action
+        { Rule.Anonymous_action.action
         ; loc
         ; dir = Alias.dir t
         ; alias = Some (Alias.name t)
@@ -175,7 +176,7 @@ let of_dir_rules ~dir rules =
 
 let of_rules rules =
   List.fold_left rules ~init:Path.Build.Map.empty ~f:(fun acc rule ->
-    Path.Build.Map.update acc rule.Rule.dir ~f:(function
+    Path.Build.Map.update acc rule.Rule.targets.root ~f:(function
       | None -> Some (Dir_rules.Nonempty.singleton (Rule rule))
       | Some acc -> Some (Dir_rules.Nonempty.add acc (Rule rule))))
 ;;
@@ -190,7 +191,8 @@ let directory_targets (rules : t) =
         match data with
         | Alias _ -> acc
         | Rule rule ->
-          Path.Build.Set.fold ~init:acc rule.targets.dirs ~f:(fun target acc ->
+          Filename.Set.fold ~init:acc rule.targets.dirs ~f:(fun target acc ->
+            let target = Path.Build.relative rule.targets.root target in
             Path.Build.Map.add_exn acc target rule.loc)))
 ;;
 
