@@ -14,6 +14,9 @@ module Blang = Blang
 module Binary_kind = Binary_kind
 module Package_constraint = Package_constraint
 module Package_name = Package_name
+module Package_dependency = Package_dependency
+module Package_version = Package_version
+module Package_variable_name = Package_variable_name
 module Lib_kind = Lib_kind
 module Lib_dep = Lib_dep
 module Pkg = Pkg
@@ -23,7 +26,11 @@ module Bindings = Bindings
 module Subst_config = Subst_config
 module Locks = Locks
 module Profile = Profile
+module Slang = Slang
 module Targets_spec = Targets_spec
 module Wrapped = Wrapped
 module Visibility = Visibility
 module Dep_conf = Dep_conf
+module Relop = Relop
+module Cond = Cond
+module Toggle = Toggle
