@@ -16,13 +16,13 @@ module Var : sig
         | Doc
         | Stublibs
         | Man
-        | Misc
 
       val of_string : string -> t option
     end
 
     type t =
       | Switch
+      | Os
       | Os_version
       | Os_distribution
       | Os_family
@@ -32,6 +32,7 @@ module Var : sig
       | Group
       | Jobs
       | Arch
+      | Sys_ocaml_version
       | Section_dir of Section.t
 
     val compare : t -> t -> Ordering.t
