@@ -1,7 +1,7 @@
 type 'a t
 
-val create :
-     lock_timeout:float option
+val create
+  :  lock_timeout:float option
   -> registry:[ `Add | `Skip ]
   -> root:string
   -> watch_mode_config:Watch_mode_config.t
@@ -12,8 +12,7 @@ val create :
   -> parse_build:(string -> 'a)
   -> 'a t
 
-type 'a pending_build_action =
-  | Build of 'a list * Decl.Build_outcome.t Fiber.Ivar.t
+type 'a pending_build_action = Build of 'a list * Decl.Build_outcome.t Fiber.Ivar.t
 
 val pending_build_action : 'a t -> 'a pending_build_action Fiber.t
 
@@ -22,7 +21,5 @@ val pending_build_action : 'a t -> 'a pending_build_action Fiber.t
 val stop : _ t -> unit Fiber.t
 
 val ready : _ t -> unit Fiber.t
-
 val run : _ t -> unit Fiber.t
-
 val action_runner : _ t -> Dune_engine.Action_runner.Rpc_server.t
