@@ -4,17 +4,20 @@ open Memo.O
 let ocaml_flags t ~dir (spec : Ocaml_flags.Spec.t) =
   let* expander = Super_context.expander t ~dir in
   let* flags =
-    let+ ocaml_flags = Super_context.env_node t ~dir >>= Env_node.ocaml_flags in
+    let+ ocaml_flags = Ocaml_flags_db.ocaml_flags_env ~dir in
     Ocaml_flags.make
       ~spec
       ~default:ocaml_flags
       ~eval:(Expander.expand_and_eval_set expander)
   in
   Source_tree.is_vendored (Path.Build.drop_build_context_exn dir)
-  >>| function
-  | false -> flags
+  >>= function
+  | false -> Memo.return flags
   | true ->
-    let ocaml_version = (Super_context.context t |> Context.ocaml).version in
+    let+ ocaml_version =
+      let+ ocaml = Super_context.context t |> Context.ocaml in
+      ocaml.version
+    in
     Ocaml_flags.with_vendored_flags ~ocaml_version flags
 ;;
 
@@ -52,8 +55,8 @@ let with_lib_deps (t : Context.t) compile_info ~dir ~f =
 ;;
 
 type kind =
-  | Executables of Dune_file.Buildable.t * (Loc.t * string) list
-  | Library of Dune_file.Buildable.t * Lib_name.Local.t
+  | Executables of Buildable.t * (Loc.t * string) list
+  | Library of Buildable.t * Lib_name.Local.t
   | Melange of
       { preprocess : Preprocess.With_instrumentation.t Preprocess.Per_module.t
       ; preprocessor_deps : Dep_conf.t list
@@ -81,7 +84,7 @@ let modules_rules
       Resolve.Memo.read_memo
         (Preprocess.Per_module.with_instrumentation preprocess ~instrumentation_backend)
     in
-    let+ instrumentation_deps =
+    let* instrumentation_deps =
       (* TODO wrong and blocks loading all the rules in this directory *)
       Resolve.Memo.read_memo
         (Preprocess.Per_module.instrumentation_deps preprocess ~instrumentation_backend)
