@@ -16,6 +16,9 @@ val capture_outputs : bool ref
 (** Print debug info about artifact substitution *)
 val debug_artifact_substitution : bool ref
 
+(** Whether we are ignoring "dune.lock/". *)
+val ignore_lock_dir : bool ref
+
 type on_missing_dune_project_file =
   | Error
   | Warn
