@@ -6,8 +6,8 @@ open Import
 
 type t
 
-val make :
-     ?excluded_modules:Module_name.t list
+val make
+  :  ?excluded_modules:Module_name.t list
   -> obj_dir:Path.Build.t Obj_dir.t
   -> modules:Modules.t
   -> top_sorted_modules:Module.t list Action_builder.t
@@ -16,8 +16,5 @@ val make :
   -> t
 
 val unsorted_objects_and_cms : t -> mode:Mode.t -> Path.t list
-
 val top_sorted_cms : t -> mode:Mode.t -> Path.t list Action_builder.t
-
-val top_sorted_objects_and_cms :
-  t -> mode:Mode.t -> Path.t list Action_builder.t
+val top_sorted_objects_and_cms : t -> mode:Mode.t -> Path.t list Action_builder.t
