@@ -30,8 +30,8 @@ end
 
 module Env_nodes : sig
   type t =
-    { context : Dune_env.Stanza.t option
-    ; workspace : Dune_env.Stanza.t option
+    { context : Dune_env.t option
+    ; workspace : Dune_env.t option
     }
 end
 
@@ -41,12 +41,12 @@ end
     potential dependency cycle. *)
 type t
 
-val ocaml : t -> Ocaml_toolchain.t
+val ocaml : t -> Ocaml_toolchain.t Memo.t
 val build_context : t -> Build_context.t
 val kind : t -> Kind.t
-val findlib_paths : t -> Path.t list
+val findlib_paths : t -> Path.t list Memo.t
 val installed_env : t -> Env.t
-val default_ocamlpath : t -> Path.t list
+val default_ocamlpath : t -> Path.t list Memo.t
 val findlib_toolchain : t -> Context_name.t option
 val instrument_with : t -> Lib_name.t list
 val profile : t -> Profile.t
@@ -67,7 +67,7 @@ val fdo_target_exe : t -> Path.t option
     adding (disable_dynamically_linked_foreign_archives true) to the workspace
     file, in which case bytecode executables will be built with all foreign
     archives statically linked into the runtime system. *)
-val dynamically_linked_foreign_archives : t -> bool
+val dynamically_linked_foreign_archives : t -> bool Memo.t
 
 (** [env_nodes] env nodes to initialize this context with. *)
 val env_nodes : t -> Env_nodes.t
@@ -78,7 +78,7 @@ val path : t -> Path.t list
 (** If this context is a cross-compilation context, you need another
     context for building tools used for the compilation that run on the
     host. *)
-val for_host : t -> t option
+val for_host : t -> t Memo.t option
 
 (** Directory where artifact are stored, for instance "_build/default" *)
 val build_dir : t -> Path.Build.t
@@ -100,7 +100,7 @@ val map_exe : t -> Path.t -> Path.t
     an explicit installation directory. *)
 val roots : t -> Path.t option Install.Roots.t
 
-val host : t -> t
+val host : t -> t Memo.t
 
 module DB : sig
   val get : Context_name.t -> t Memo.t
