@@ -6,8 +6,8 @@ end
 
 val action : Context.t -> src:Path.t -> dst:Path.Build.t -> Action.t
 
-val builder :
-     Context.t
+val builder
+  :  Context.t
   -> src:Path.t
   -> dst:Path.Build.t
   -> Action.Full.t Action_builder.With_targets.t
