@@ -8,13 +8,9 @@ module Name : sig
   type t
 
   val make : string -> t
-
   val compare : t -> t -> Ordering.t
-
   val equal : t -> t -> bool
-
   val to_dyn : t -> Dyn.t
-
   val to_string : t -> string
 end
 
@@ -23,9 +19,8 @@ type t
 module Map : Map.S with type key = t
 
 (** A Coq module [a.b.foo] defined in file [a/b/foo.v] *)
-val make :
-     source:Path.t
-       (** file = .v source file; module name has to be the same so far *)
+val make
+  :  source:Path.t (** file = .v source file; module name has to be the same so far *)
   -> prefix:string list (** Library-local qualified prefix *)
   -> name:Name.t (** Name of the module *)
   -> t
@@ -33,13 +28,9 @@ val make :
 (** Coq does enforce some invariants wrt module vs file names *)
 
 val source : t -> Path.t
-
 val prefix : t -> string list
-
 val name : t -> Name.t
-
 val dep_file : t -> obj_dir:Path.Build.t -> Path.Build.t
-
 val glob_file : t -> obj_dir:Path.Build.t -> Path.Build.t
 
 (** Some of the object files should not be installed, we control this with the
@@ -50,8 +41,8 @@ type obj_files_mode =
 
 (** This returns a list of pairs [(obj_file, install_path)] due to native files
     having a different install location *)
-val obj_files :
-     t
+val obj_files
+  :  t
   -> wrapper_name:string
   -> mode:Coq_mode.t
   -> obj_dir:Path.Build.t
@@ -59,5 +50,4 @@ val obj_files :
   -> (Path.Build.t * string) list
 
 val to_dyn : t -> Dyn.t
-
 val eval : dir:Path.Build.t -> standard:t list -> Ordered_set_lang.t -> t list
