@@ -6,5 +6,5 @@ val rules
   :  sctx:Super_context.t
   -> expander:Expander.t
   -> dir:Path.Build.t
-  -> (Cram_test.t, Source_tree.Dir.error) result list
+  -> Source_tree.Dir.t
   -> unit Memo.t
