@@ -54,9 +54,6 @@ open Import
    generation. As a result, there are no intermediate libraries (or
    packages). *)
 
-module Buildable = Dune_file.Buildable
-module Library = Dune_file.Library
-
 let verbatimf fmt = Printf.ksprintf (fun s -> Pp.concat [ Pp.verbatim s; Pp.newline ]) fmt
 
 let write_c_types_includer_module ~type_description_functor ~c_generated_types_module =
@@ -179,16 +176,20 @@ let build_c_program
   ~deps
   =
   let ctx = Super_context.context sctx in
-  let open Memo.O in
-  let* exe =
-    Ocaml_config.c_compiler (Context.ocaml ctx).ocaml_config
+  let ocaml = Context.ocaml ctx in
+  let exe =
+    let open Action_builder.O in
+    let* ocaml = Action_builder.of_memo ocaml in
+    Ocaml_config.c_compiler ocaml.ocaml_config
     |> Super_context.resolve_program ~loc:None ~dir sctx
   in
   let project = Scope.project scope in
   let with_user_and_std_flags =
     let base_flags =
+      let open Action_builder.O in
+      let+ ocaml = Action_builder.of_memo ocaml in
       let use_standard_flags = Dune_project.use_standard_c_and_cxx_flags project in
-      let cfg = (Context.ocaml ctx).ocaml_config in
+      let cfg = ocaml.ocaml_config in
       let fdo_flags = Command.Args.As (Fdo.c_flags ctx) in
       match use_standard_flags with
       | Some true -> fdo_flags
@@ -210,10 +211,12 @@ let build_c_program
         ~flags:Ordered_set_lang.Unexpanded.standard
         ~language:C
     in
-    Command.Args.S [ base_flags; As foreign_flags ]
+    Command.Args.S [ Dyn base_flags; As foreign_flags ]
   in
   let include_args =
-    let ocaml_where = (Context.ocaml ctx).lib_config.stdlib_dir in
+    let open Action_builder.O in
+    let* ocaml = Action_builder.of_memo ocaml in
+    let ocaml_where = ocaml.lib_config.stdlib_dir in
     (* XXX: need glob dependency *)
     let open Action_builder.O in
     let ctypes = Lib_name.of_string "ctypes" in
@@ -256,7 +259,8 @@ let build_c_program
       ]
     in
     let open Action_builder.With_targets.O in
-    Action_builder.with_no_targets deps >>> Command.run ~dir:(Path.build dir) exe args
+    Action_builder.with_no_targets deps
+    >>> Command.run_dyn_prog ~dir:(Path.build dir) exe args
   in
   Super_context.add_rule sctx ~dir action
 ;;
@@ -293,8 +297,9 @@ let gen_rules ~cctx ~(buildable : Buildable.t) ~loc ~scope ~dir ~sctx =
   let c_generated_types_module = Ctypes_field.c_generated_types_module ctypes in
   let open Memo.O in
   let foreign_archives_deps =
-    let ctx = Super_context.context sctx in
-    let { Lib_config.ext_lib; ext_dll; _ } = (Context.ocaml ctx).lib_config in
+    let { Lib_config.ext_lib; ext_dll; _ } =
+      (Compilation_context.ocaml cctx).lib_config
+    in
     List.concat_map buildable.foreign_archives ~f:(fun (_loc, archive) ->
       let mode = Mode.Select.All in
       [ Foreign.Archive.lib_file ~mode ~archive ~dir ~ext_lib
