@@ -6,31 +6,69 @@ module Vfile = Dune_lang.Versioned_file.Make (struct
 
 let fn = "dune-package"
 
+module External_location = struct
+  type t =
+    | Relative_to_stdlib of Path.Local.t
+    | Relative_to_findlib of (Path.t * Path.Local.t)
+    | Absolute of Path.t
+
+  let to_dyn x =
+    let open Dyn in
+    match x with
+    | Relative_to_stdlib p -> variant "Relative_to_stdlib" [ Path.Local.to_dyn p ]
+    | Relative_to_findlib (p1, p2) ->
+      variant "Relative_to_findlib" [ pair Path.to_dyn Path.Local.to_dyn (p1, p2) ]
+    | Absolute p -> variant "Absolute" [ Path.to_dyn p ]
+  ;;
+
+  let compare x y =
+    match x, y with
+    | Relative_to_stdlib x, Relative_to_stdlib y -> Path.Local.compare x y
+    | Relative_to_findlib (x1, x2), Relative_to_findlib (y1, y2) ->
+      let open Ordering.O in
+      let= () = Path.compare x1 y1 in
+      Path.Local.compare x2 y2
+    | Absolute x, Absolute y -> Path.compare x y
+    | Relative_to_stdlib _, _ -> Lt
+    | _, Relative_to_stdlib _ -> Gt
+    | Relative_to_findlib _, Absolute _ -> Lt
+    | Absolute _, Relative_to_findlib _ -> Gt
+  ;;
+
+  let hash = Poly.hash
+end
+
 module Lib = struct
   type t =
     { info : Path.t Lib_info.t
     ; main_module_name : Module_name.t option
+    ; external_location : External_location.t option
     }
 
-  let make ~info ~main_module_name =
+  let make ~info ~main_module_name ~external_location =
     let obj_dir = Lib_info.obj_dir info in
     let dir = Obj_dir.dir obj_dir in
     let map_path p =
       if Path.is_managed p then Path.relative dir (Path.basename p) else p
     in
     let info = Lib_info.map_path info ~f:map_path in
-    { info; main_module_name }
+    { info; main_module_name; external_location }
   ;;
 
-  let of_dune_lib ~info ~main_module_name = make ~info ~main_module_name
-  let of_findlib info = make ~info ~main_module_name:None
+  let of_dune_lib ~info ~main_module_name =
+    make ~info ~main_module_name ~external_location:None
+  ;;
+
+  let of_findlib info external_location =
+    make ~info ~main_module_name:None ~external_location:(Some external_location)
+  ;;
 
   let dir_of_name name =
     let _, components = Lib_name.split name in
     Path.Local.L.relative Path.Local.root components
   ;;
 
-  let encode ~package_root ~stublibs { info; main_module_name } =
+  let encode ~package_root ~stublibs { info; main_module_name; external_location = _ } =
     let open Dune_lang.Encoder in
     let no_loc f (_loc, x) = f x in
     let path = Dune_lang.Path.Local.encode ~dir:package_root in
@@ -185,7 +223,7 @@ module Lib = struct
        and+ requires = field_l "requires" (Lib_dep.decode ~allow_re_export:true)
        and+ ppx_runtime_deps = libs "ppx_runtime_deps"
        and+ virtual_ = field_b "virtual"
-       and+ sub_systems = Sub_system_info.record_parser ()
+       and+ sub_systems = Sub_system_info.record_parser
        and+ orig_src_dir = field_o "orig_src_dir" path
        and+ modules = field "modules" (Modules.decode ~src_dir:base)
        and+ special_builtin_support =
@@ -259,7 +297,17 @@ module Lib = struct
            ~instrumentation_backend
            ~melange_runtime_deps
        in
-       { info; main_module_name })
+       let external_location =
+         let opam_dir = Path.parent_exn base in
+         let pkg, components = Lib_name.split name in
+         let local =
+           Path.Local.L.relative
+             (Path.Local.of_string (Package.Name.to_string pkg))
+             components
+         in
+         Some (External_location.Relative_to_findlib (opam_dir, local))
+       in
+       { info; main_module_name; external_location })
   ;;
 
   let main_module_name t = t.main_module_name
@@ -271,12 +319,14 @@ module Lib = struct
   ;;
 
   let info dp = dp.info
+  let external_location dp = dp.external_location
 
-  let to_dyn { info; main_module_name } =
+  let to_dyn { info; main_module_name; external_location } =
     let open Dyn in
     record
       [ "info", Lib_info.to_dyn Path.to_dyn info
       ; "main_module_name", option Module_name.to_dyn main_module_name
+      ; "external_location", option External_location.to_dyn external_location
       ]
   ;;
 end
@@ -357,20 +407,47 @@ module Entry = struct
   ;;
 end
 
+type path = [ `File | `Dir ] * Install.Entry.Dst.t
+
+let decode_path =
+  let open Dune_lang.Decoder in
+  peek_exn
+  >>= function
+  | List _ ->
+    enter
+    @@
+    let* () = keyword "dir" in
+    let+ d = Install.Entry.Dst.decode in
+    `Dir, d
+  | _ ->
+    let+ f = Install.Entry.Dst.decode in
+    `File, f
+;;
+
+let encode_path = function
+  | `File, f -> Install.Entry.Dst.encode f
+  | `Dir, d -> Dune_lang.Encoder.constr "dir" Install.Entry.Dst.encode d
+;;
+
+let path_to_dyn = function
+  | `File, f -> Install.Entry.Dst.to_dyn f
+  | `Dir, d -> Dyn.variant "dir" [ Install.Entry.Dst.to_dyn d ]
+;;
+
 type t =
   { name : Package.Name.t
   ; entries : Entry.t Lib_name.Map.t
-  ; version : string option
+  ; version : Package_version.t option
   ; sections : Path.t Section.Map.t
   ; sites : Section.t Site.Map.t
   ; dir : Path.t
-  ; files : (Section.t * Install.Entry.Dst.t list) list
+  ; files : (Section.t * path list) list
   }
 
 let decode ~lang ~dir =
   let open Dune_lang.Decoder in
   let+ name = field "name" Package.Name.decode
-  and+ version = field_o "version" string
+  and+ version = field_o "version" Package_version.decode
   and+ sections =
     field
       ~default:[]
@@ -379,10 +456,7 @@ let decode ~lang ~dir =
   and+ sites =
     field ~default:[] "sites" (repeat (pair (located Site.decode) Section.decode))
   and+ files =
-    field
-      ~default:[]
-      "files"
-      (repeat (pair Section.decode (enter (repeat Install.Entry.Dst.decode))))
+    field ~default:[] "files" (repeat (pair Section.decode (enter (repeat decode_path))))
   and+ entries = leftover_fields_as_sums (Entry.cstrs ~lang ~dir) in
   let entries =
     List.map entries ~f:(fun e ->
@@ -436,13 +510,13 @@ let encode ~dune_version { entries; name; version; dir; sections; sites; files }
   let sexp =
     record_fields
       [ field "name" Package.Name.encode name
-      ; field_o "version" string version
+      ; field_o "version" Package_version.encode version
       ; field_l
           "sections"
           (pair Section.encode (Dune_lang.Path.Local.encode ~dir))
           (Section.Map.to_list sections)
       ; field_l "sites" (pair Site.encode Section.encode) sites
-      ; field_l "files" (pair Section.encode (list Install.Entry.Dst.encode)) files
+      ; field_l "files" (pair Section.encode (list encode_path)) files
       ]
   in
   let list s = Dune_lang.List s in
@@ -467,11 +541,11 @@ let to_dyn { entries; name; version; dir; sections; sites; files } =
   record
     [ "entries", list Entry.to_dyn (Lib_name.Map.values entries)
     ; "name", Package.Name.to_dyn name
-    ; "version", option string version
+    ; "version", option Package_version.to_dyn version
     ; "dir", Path.to_dyn dir
     ; "sections", Section.Map.to_dyn Path.to_dyn sections
     ; "sites", Site.Map.to_dyn Section.to_dyn sites
-    ; "files", (list (pair Section.to_dyn (list Install.Entry.Dst.to_dyn))) files
+    ; "files", (list (pair Section.to_dyn (list path_to_dyn))) files
     ]
 ;;
 
