@@ -5,6 +5,17 @@ open Import
 (** The filename of a dune-package file*)
 val fn : string
 
+module External_location : sig
+  type t =
+    | Relative_to_stdlib of Path.Local.t
+    | Relative_to_findlib of (Path.t * Path.Local.t)
+    | Absolute of Path.t
+
+  val to_dyn : t Dyn.builder
+  val compare : t -> t -> Ordering.t
+  val hash : t -> int
+end
+
 module Lib : sig
   type t
 
@@ -12,7 +23,8 @@ module Lib : sig
   val dir_of_name : Lib_name.t -> Path.Local.t
   val wrapped : t -> Wrapped.t option
   val info : t -> Path.t Lib_info.t
-  val of_findlib : Path.t Lib_info.t -> t
+  val external_location : t -> External_location.t option
+  val of_findlib : Path.t Lib_info.t -> External_location.t -> t
   val of_dune_lib : info:Path.t Lib_info.t -> main_module_name:Module_name.t option -> t
   val to_dyn : t Dyn.builder
 end
@@ -32,28 +44,30 @@ module Entry : sig
     | Library of Lib.t
     | Deprecated_library_name of Deprecated_library_name.t
     | Hidden_library of Lib.t
-        (** Only for external libraries that:
+    (** Only for external libraries that:
 
-            - are not built with dune
+        - are not built with dune
 
-            - have a [META] file with an unsatisfied [exist_if] field
+        - have a [META] file with an unsatisfied [exist_if] field
 
-            Dune itself never produces hidden libraries. *)
+        Dune itself never produces hidden libraries. *)
 
   val name : t -> Lib_name.t
-  val version : t -> string option
+  val version : t -> Package_version.t option
   val loc : t -> Loc.t
   val to_dyn : t Dyn.builder
 end
 
+type path = [ `File | `Dir ] * Install.Entry.Dst.t
+
 type t =
   { name : Package.Name.t
   ; entries : Entry.t Lib_name.Map.t
-  ; version : string option
+  ; version : Package_version.t option
   ; sections : Path.t Section.Map.t
   ; sites : Section.t Site.Map.t
   ; dir : Path.t
-  ; files : (Section.t * Install.Entry.Dst.t list) list
+  ; files : (Section.t * path list) list
   }
 
 val to_dyn : t Dyn.builder
