@@ -9,6 +9,6 @@ val gen_rules : Super_context.t -> output_dir:Path.Build.t -> unit Memo.t
 
 (** This must be called from the main directory, i.e. the ones containing the
     source files and the the [formatted_dir_basename] sub-directory. *)
-val setup_alias : Super_context.t -> dir:Path.Build.t -> unit Memo.t
+val setup_alias : dir:Path.Build.t -> unit Memo.t
 
 val formatted_dir_basename : Filename.t
