@@ -74,7 +74,7 @@ let eval ~loc ~expander ~paths:path_spec (deps : Dep_conf.t list) =
   Option.iter sandbox ~f:(fun _ ->
     User_error.raise ~loc [ Pp.text "sandbox settings are not allowed" ]);
   let open Memo.O in
-  let+ paths, _ = Action_builder.run runtime_deps Lazy in
+  let+ paths, _ = Action_builder.evaluate_and_collect_deps runtime_deps in
   (match path_spec with
    | Allow_all -> ()
    | Disallow_external lib_name ->
