@@ -9,8 +9,8 @@ module Lib_and_module : sig
   module L : sig
     type nonrec t = t list
 
-    val link_flags :
-         Super_context.t
+    val link_flags
+      :  Super_context.t
       -> t
       -> lib_config:Lib_config.t
       -> mode:Link_mode.t
@@ -23,14 +23,10 @@ module L : sig
   type nonrec t = Lib.t list
 
   val to_iflags : Path.Set.t -> _ Command.Args.t
-
   val include_paths : ?project:Dune_project.t -> t -> Lib_mode.t -> Path.Set.t
-
-  val include_flags :
-    ?project:Dune_project.t -> t -> Lib_mode.t -> _ Command.Args.t
-
+  val include_flags : ?project:Dune_project.t -> t -> Lib_mode.t -> _ Command.Args.t
   val c_include_flags : t -> Super_context.t -> _ Command.Args.t
-
+  val toplevel_ld_paths : t -> Path.Set.t
   val toplevel_include_paths : t -> Path.Set.t
 end
 
