@@ -1,9 +1,5 @@
 open Import
 open Memo.O
-module Buildable = Dune_file.Buildable
-module Library = Dune_file.Library
-module Public_lib = Dune_file.Public_lib
-module Mode_conf = Dune_file.Mode_conf
 
 let msvc_hack_cclibs =
   List.map ~f:(fun lib ->
@@ -27,7 +23,8 @@ let build_lib
   ~cm_files
   =
   let ctx = Super_context.context sctx in
-  Ocaml_toolchain.compiler (Context.ocaml ctx) mode
+  let* ocaml = Context.ocaml ctx in
+  Ocaml_toolchain.compiler ocaml mode
   |> Memo.Result.iter ~f:(fun compiler ->
     let target = Library.archive lib ~dir ~ext:(Mode.compiled_lib_ext mode) in
     let stubs_flags =
@@ -55,9 +52,10 @@ let build_lib
       | Some lib_archive -> make_args ~stub_mode lib_archive @ foreign_archives
       | None -> foreign_archives
     in
+    let* ocaml = Context.ocaml ctx in
     let map_cclibs =
       (* https://github.com/ocaml/dune/issues/119 *)
-      match (Context.ocaml ctx).lib_config.ccomp_type with
+      match ocaml.lib_config.ccomp_type with
       | Msvc -> msvc_hack_cclibs
       | Other _ -> Fun.id
     in
@@ -66,7 +64,7 @@ let build_lib
     in
     let ocaml_flags = Ocaml_flags.get flags (Ocaml mode) in
     let* standard =
-      let+ project = Scope.DB.find_by_dir dir |> Memo.map ~f:Scope.project in
+      let+ project = Dune_load.find_project ~dir in
       match Dune_project.use_standard_c_and_cxx_flags project with
       | Some true when Buildable.has_foreign_cxx lib.buildable ->
         Cxx_flags.get_flags ~for_:Link (Context.build_context ctx)
@@ -115,7 +113,7 @@ let build_lib
              ; Deps
                  (Foreign.Objects.build_paths
                     lib.buildable.extra_objects
-                    ~ext_obj:(Context.ocaml ctx).lib_config.ext_obj
+                    ~ext_obj:ocaml.lib_config.ext_obj
                     ~dir)
              ]))
 ;;
@@ -164,7 +162,8 @@ let ocamlmklib
   ~build_targets_together
   =
   let ctx = Super_context.context sctx in
-  let { Lib_config.ext_lib; ext_dll; _ } = (Context.ocaml ctx).lib_config in
+  let* ocaml = Context.ocaml ctx in
+  let { Lib_config.ext_lib; ext_dll; _ } = ocaml.lib_config in
   let static_target =
     Foreign.Archive.Name.lib_file archive_name ~dir ~ext_lib ~mode:stubs_mode
   in
@@ -172,7 +171,7 @@ let ocamlmklib
     Action_builder.map c_library_flags ~f:(fun cclibs ->
       (* https://github.com/ocaml/dune/issues/119 *)
       let cclibs =
-        match (Context.ocaml ctx).lib_config.ccomp_type with
+        match ocaml.lib_config.ccomp_type with
         | Msvc -> msvc_hack_cclibs cclibs
         | Other _ -> cclibs
       in
@@ -187,7 +186,7 @@ let ocamlmklib
        let ctx = Super_context.context sctx in
        Command.run
          ~dir:(Path.build (Context.build_dir ctx))
-         (Context.ocaml ctx).ocamlmklib
+         ocaml.ocamlmklib
          [ A "-g"
          ; (if custom then A "-custom" else Command.Args.empty)
          ; A "-o"
@@ -208,12 +207,11 @@ let ocamlmklib
   then
     (* Build both the static and dynamic targets in one [ocamlmklib] invocation,
        unless dynamically linked foreign archives are disabled. *)
-    build
-      ~sandbox:Sandbox_config.no_special_requirements
-      ~custom:false
-      (if Context.dynamically_linked_foreign_archives ctx
-       then [ static_target; dynamic_target ]
-       else [ static_target ])
+    Context.dynamically_linked_foreign_archives ctx
+    >>| (function
+           | true -> [ static_target; dynamic_target ]
+           | false -> [ static_target ])
+    >>= build ~sandbox:Sandbox_config.no_special_requirements ~custom:false
   else
     let open Memo.O in
     (* Build the static target only by passing the [-custom] flag. *)
@@ -231,7 +229,10 @@ let ocamlmklib
        "optional targets", allowing us to run [ocamlmklib] with the [-failsafe]
        flag, which always produces the static target and sometimes produces the
        dynamic target too. *)
-    Memo.when_ (Context.dynamically_linked_foreign_archives ctx) (fun () ->
+    let* dynamically_linked_foreign_archives =
+      Context.dynamically_linked_foreign_archives ctx
+    in
+    Memo.when_ dynamically_linked_foreign_archives (fun () ->
       build ~sandbox:Sandbox_config.needs_sandboxing ~custom:false [ dynamic_target ])
 ;;
 
@@ -258,13 +259,10 @@ let foreign_rules (library : Foreign.Library.t) ~sctx ~expander ~dir ~dir_conten
   in
   let* () = Check_rules.add_files sctx ~dir o_files in
   let* standard =
-    let+ project =
-      let+ scope = Scope.DB.find_by_dir dir in
-      Scope.project scope
-    in
-    let ctx = Super_context.context sctx in
+    let+ project = Dune_load.find_project ~dir in
     match Dune_project.use_standard_c_and_cxx_flags project with
     | Some true when Foreign.Sources.has_cxx_sources foreign_sources ->
+      let ctx = Super_context.context sctx in
       Cxx_flags.get_flags ~for_:Link (Context.build_context ctx)
     | _ -> Action_builder.return []
   in
@@ -292,9 +290,7 @@ let build_stubs lib ~cctx ~dir ~expander ~requires ~dir_contents ~vlib_stubs_o_f
   in
   let* o_files =
     let lib_foreign_o_files =
-      let { Lib_config.ext_obj; _ } =
-        (Super_context.context sctx |> Context.ocaml).lib_config
-      in
+      let { Lib_config.ext_obj; _ } = (Compilation_context.ocaml cctx).lib_config in
       Foreign.Objects.build_paths lib.buildable.extra_objects ~ext_obj ~dir
     in
     let+ tbl =
@@ -320,10 +316,12 @@ let build_stubs lib ~cctx ~dir ~expander ~requires ~dir_contents ~vlib_stubs_o_f
     let build_targets_together =
       modes.ocaml.native
       && modes.ocaml.byte
-      && Dynlink_supported.get_ocaml_config lib.dynlink (Context.ocaml ctx).ocaml_config
+      && Dynlink_supported.get_ocaml_config
+           lib.dynlink
+           (Compilation_context.ocaml cctx).ocaml_config
     in
     let* standard =
-      let+ project = Scope.DB.find_by_dir dir >>| Scope.project in
+      let+ project = Dune_load.find_project ~dir in
       match Dune_project.use_standard_c_and_cxx_flags project with
       | Some true when Foreign.Sources.has_cxx_sources foreign_sources ->
         Cxx_flags.get_flags ~for_:Link (Context.build_context ctx)
@@ -370,8 +368,9 @@ let build_stubs lib ~cctx ~dir ~expander ~requires ~dir_contents ~vlib_stubs_o_f
 
 let build_shared lib ~native_archives ~sctx ~dir ~flags =
   let ctx = Super_context.context sctx in
-  Memo.Result.iter (Context.ocaml ctx).ocamlopt ~f:(fun ocamlopt ->
-    let ext_lib = (Context.ocaml ctx).lib_config.ext_lib in
+  let* ocaml = Context.ocaml ctx in
+  Memo.Result.iter ocaml.ocamlopt ~f:(fun ocamlopt ->
+    let ext_lib = ocaml.lib_config.ext_lib in
     let src =
       let ext = Mode.compiled_lib_ext Native in
       Path.build (Library.archive lib ~dir ~ext)
@@ -422,21 +421,15 @@ let iter_modes_concurrently (t : _ Ocaml.Mode.Dict.t) ~(f : Ocaml.Mode.t -> unit
   ()
 ;;
 
-let setup_build_archives
-  (lib : Dune_file.Library.t)
-  ~top_sorted_modules
-  ~cctx
-  ~expander
-  ~lib_info
-  =
+let setup_build_archives (lib : Library.t) ~top_sorted_modules ~cctx ~expander ~lib_info =
   let obj_dir = Compilation_context.obj_dir cctx in
   let dir = Compilation_context.dir cctx in
   let flags = Compilation_context.flags cctx in
   let modules = Compilation_context.modules cctx in
   let js_of_ocaml = Js_of_ocaml.In_context.make ~dir lib.buildable.js_of_ocaml in
   let sctx = Compilation_context.super_context cctx in
-  let ctx = Compilation_context.context cctx in
-  let { Lib_config.ext_obj; natdynlink_supported; _ } = (Context.ocaml ctx).lib_config in
+  let ocaml = Compilation_context.ocaml cctx in
+  let { Lib_config.ext_obj; natdynlink_supported; _ } = ocaml.lib_config in
   let open Memo.O in
   let* () =
     Modules.exit_module modules
@@ -495,7 +488,7 @@ let setup_build_archives
             ~obj_dir)
       in
       Memo.parallel_iter action_with_targets ~f:(fun rule ->
-        rule >>= Super_context.add_rule sctx ~dir ~loc:lib.buildable.loc))
+        Super_context.add_rule sctx ~dir ~loc:lib.buildable.loc rule))
   in
   Memo.when_
     (Dynlink_supported.By_the_os.get natdynlink_supported && modes.ocaml.native)
@@ -506,7 +499,10 @@ let cctx (lib : Library.t) ~sctx ~source_modules ~dir ~expander ~scope ~compile_
   let* flags = Buildable_rules.ocaml_flags sctx ~dir lib.buildable.flags
   and* vimpl = Virtual_rules.impl sctx ~lib ~scope in
   let obj_dir = Library.obj_dir ~dir lib in
-  let ctx = Super_context.context sctx in
+  let* ocaml =
+    let ctx = Super_context.context sctx in
+    Context.ocaml ctx
+  in
   let* modules, pp =
     Buildable_rules.modules_rules
       sctx
@@ -520,10 +516,10 @@ let cctx (lib : Library.t) ~sctx ~source_modules ~dir ~expander ~scope ~compile_
   let requires_compile = Lib.Compile.direct_requires compile_info in
   let requires_link = Lib.Compile.requires_link compile_info in
   let modes =
-    let { Lib_config.has_native; _ } = (Context.ocaml ctx).lib_config in
-    Dune_file.Mode_conf.Lib.Set.eval_detailed lib.modes ~has_native
+    let { Lib_config.has_native; _ } = ocaml.lib_config in
+    Mode_conf.Lib.Set.eval_detailed lib.modes ~has_native
   in
-  let package = Dune_file.Library.package lib in
+  let package = Library.package lib in
   let js_of_ocaml = Js_of_ocaml.In_context.make ~dir lib.buildable.js_of_ocaml in
   (* XXX(anmonteiro): `public_lib_name` is used to derive Melange's
      `--bs-package-name` argument. We only use the library name for public
@@ -572,9 +568,8 @@ let library_rules
   let dir = Compilation_context.dir cctx in
   let scope = Compilation_context.scope cctx in
   let* requires_compile = Compilation_context.requires_compile cctx in
-  let stdlib_dir =
-    (Compilation_context.context cctx |> Context.ocaml).lib_config.stdlib_dir
-  in
+  let ocaml = Compilation_context.ocaml cctx in
+  let stdlib_dir = ocaml.lib_config.stdlib_dir in
   let top_sorted_modules =
     let impl_only = Modules.impl_only modules in
     Dep_graph.top_closed_implementations
@@ -589,7 +584,7 @@ let library_rules
   and* () = Module_compilation.build_all cctx
   and* expander = Super_context.expander sctx ~dir
   and* lib_info =
-    let lib_config = (Super_context.context sctx |> Context.ocaml).lib_config in
+    let lib_config = ocaml.lib_config in
     let* info = Library.to_lib_info lib ~dir ~lib_config in
     let mode = Lib_mode.Map.Set.for_merlin (Lib_info.modes info) in
     let+ () = Check_rules.add_obj_dir sctx ~obj_dir mode in
