@@ -3,8 +3,7 @@ open Import
 let () = Inline_tests.linkme
 
 type build_system =
-  { conf : Dune_load.conf
-  ; contexts : Context.t list
+  { contexts : Context.t list
   ; scontexts : Super_context.t Context_name.Map.t
   }
 
@@ -15,26 +14,31 @@ let implicit_default_alias dir =
     let open Memo.O in
     Source_tree.find_dir src_dir
     >>| (function
-    | None -> None
-    | Some src_dir ->
-      let default_alias =
-        let dune_version = Source_tree.Dir.project src_dir |> Dune_project.dune_version in
-        if dune_version >= (2, 0) then Alias0.all else Alias0.install
-      in
-      Some (Action_builder.ignore (Alias_rec.dep_on_alias_rec default_alias dir)))
+     | None -> None
+     | Some src_dir ->
+       let default_alias =
+         let dune_version =
+           Source_tree.Dir.project src_dir |> Dune_project.dune_version
+         in
+         if dune_version >= (2, 0) then Alias0.all else Alias0.install
+       in
+       Some (Action_builder.ignore (Alias_rec.dep_on_alias_rec default_alias dir)))
 ;;
 
 let execution_parameters =
   let f path =
-    let open Memo.O in
-    let+ dir = Source_tree.nearest_dir path
-    and+ ep = Execution_parameters.default in
-    Dune_project.update_execution_parameters (Source_tree.Dir.project dir) ep
+    match Path.Build.drop_build_context path with
+    | None -> Dune_engine.Execution_parameters.default
+    | Some path ->
+      let open Memo.O in
+      let+ dir = Source_tree.nearest_dir path
+      and+ ep = Execution_parameters.default in
+      Dune_project.update_execution_parameters (Source_tree.Dir.project dir) ep
   in
   let memo =
     Memo.create
       "execution-parameters-of-dir"
-      ~input:(module Path.Source)
+      ~input:(module Path.Build)
       ~cutoff:Execution_parameters.equal
       f
   in
@@ -51,12 +55,9 @@ let init
   ()
   : unit
   =
-  let promote_source ~chmod ~delete_dst_if_it_is_a_directory ~src ~dst ctx =
+  let promote_source ~chmod ~delete_dst_if_it_is_a_directory ~src ~dst =
     let open Fiber.O in
-    let* ctx =
-      Memo.run
-        (Memo.Option.map ctx ~f:(fun (ctx : Build_context.t) -> Context.DB.get ctx.name))
-    in
+    let* ctx = Path.Build.parent_exn src |> Context.DB.by_dir |> Memo.run in
     let conf = Artifact_substitution.Conf.of_context ctx in
     let src = Path.build src in
     let dst = Path.source dst in
@@ -102,11 +103,10 @@ let init
 
 let get () =
   let open Memo.O in
-  let* conf = Dune_load.load () in
   let* contexts = Context.DB.all () in
   let* scontexts = Memo.Lazy.force Super_context.all in
   let* () = Super_context.all_init_deferred () in
-  Memo.return { conf; contexts; scontexts }
+  Memo.return { contexts; scontexts }
 ;;
 
 let find_context_exn t ~name =
