@@ -5,11 +5,12 @@ module Module_system = struct
     | Es6
     | CommonJs
 
-  let default = (CommonJs, ".js")
+  let default = CommonJs, ".js"
 
   let to_string = function
     | Es6 -> "es6"
     | CommonJs -> "commonjs"
+  ;;
 end
 
 module Cm_kind = struct
@@ -20,16 +21,19 @@ module Cm_kind = struct
   let source = function
     | Cmi -> Ocaml.Ml_kind.Intf
     | Cmj -> Impl
+  ;;
 
   let ext = function
     | Cmi -> ".cmi"
     | Cmj -> ".cmj"
+  ;;
 
   let to_dyn =
     let open Dyn in
     function
     | Cmi -> variant "cmi" []
     | Cmj -> variant "cmj" []
+  ;;
 
   module Map = struct
     type 'a t =
@@ -47,14 +51,17 @@ end
 
 let js_basename m =
   match Module.file ~ml_kind:Impl m with
-  | Some s -> (
+  | Some s ->
     (* we aren't using Filename.extension because we want to handle
        filenames such as foo.pp.ml *)
-    match String.lsplit2 (Path.basename s) ~on:'.' with
-    | None ->
-      Code_error.raise "could not extract module name from file path"
-        [ ("module", Module.to_dyn m) ]
-    | Some (module_name, _) -> module_name)
+    (match String.lsplit2 (Path.basename s) ~on:'.' with
+     | None ->
+       Code_error.raise
+         "could not extract module name from file path"
+         [ "module", Module.to_dyn m ]
+     | Some (module_name, _) -> module_name)
   | None ->
-    Code_error.raise "could not find melange source from module"
-      [ ("module", Module.to_dyn m) ]
+    Code_error.raise
+      "could not find melange source from module"
+      [ "module", Module.to_dyn m ]
+;;
