@@ -1,22 +1,5 @@
 open Import
-
-let ocaml_flags sctx ~dir melange =
-  let open Memo.O in
-  let* expander = Super_context.expander sctx ~dir in
-  let* flags =
-    let+ ocaml_flags = Super_context.env_node sctx ~dir >>= Env_node.ocaml_flags in
-    Ocaml_flags.make_with_melange
-      ~melange
-      ~default:ocaml_flags
-      ~eval:(Expander.expand_and_eval_set expander)
-  in
-  Source_tree.is_vendored (Path.Build.drop_build_context_exn dir)
-  >>| function
-  | false -> flags
-  | true ->
-    let ocaml_version = (Super_context.context sctx |> Context.ocaml).version in
-    Ocaml_flags.with_vendored_flags ~ocaml_version flags
-;;
+open Memo.O
 
 let output_of_lib ~target_dir lib =
   let info = Lib.info lib in
@@ -58,7 +41,6 @@ let make_js_name ~js_ext ~output m =
 ;;
 
 let impl_only_modules_defined_in_this_lib sctx lib =
-  let open Memo.O in
   let+ modules = Dir_contents.modules_of_lib sctx lib in
   match modules with
   | None ->
@@ -233,7 +215,12 @@ let setup_emit_cmj_rules
         modules
     in
     let requires_link = Lib.Compile.requires_link compile_info in
-    let* flags = ocaml_flags sctx ~dir mel.compile_flags in
+    let* flags =
+      let specific = Lib_mode.Map.make_all mel.compile_flags in
+      Ocaml_flags.Spec.make ~common:Ordered_set_lang.Unexpanded.standard ~specific
+      |> Ocaml_flags_db.ocaml_flags sctx ~dir
+      >>| Ocaml_flags.allow_only_melange
+    in
     let* cctx =
       let direct_requires = Lib.Compile.direct_requires compile_info in
       Compilation_context.create
@@ -255,7 +242,7 @@ let setup_emit_cmj_rules
     in
     let* () = Module_compilation.build_all cctx in
     let* requires_compile = Compilation_context.requires_compile cctx in
-    let stdlib_dir = (Context.ocaml ctx).lib_config.stdlib_dir in
+    let stdlib_dir = (Compilation_context.ocaml cctx).lib_config.stdlib_dir in
     let+ () =
       let emit_and_libs_deps =
         let target_dir = Path.Build.relative dir mel.target in
@@ -376,7 +363,8 @@ let modules_for_js_and_obj_dir ~sctx ~dir_contents ~scope (mel : Melange_stanzas
     >>| Ml_sources.modules_and_obj_dir ~for_:(Melange { target = mel.target })
   in
   let+ modules =
-    let version = (Super_context.context sctx |> Context.ocaml).version in
+    let* ocaml = Context.ocaml (Super_context.context sctx) in
+    let version = ocaml.version in
     let* preprocess =
       Resolve.Memo.read_memo
         (Preprocess.Per_module.with_instrumentation
@@ -557,3 +545,106 @@ let setup_emit_js_rules ~dir_contents ~dir ~scope ~sctx mel =
              { fail = (fun () -> Resolve.raise_error_with_stack_trace resolve_error) }
            |> Action_builder.with_file_targets ~file_targets)))
 ;;
+
+(* The emit stanza of melange outputs in a single output directory (and its
+   descendants). We attach all .js generating rules to this root directory.
+
+   Since we allow user defined rules in this output directory, we need to know
+   when we're under the emit directory so that we load both the user defined
+   rules and the rules originating from the emit stanza. *)
+type t =
+  { (* the directory in which the emit stanza is defined. *)
+    stanza_dir : Path.Build.t
+  ; (* the emit stanza itself. *)
+    stanza : Melange_stanzas.Emit.t
+  }
+
+let emit_rules sctx { stanza_dir; stanza } =
+  Rules.collect_unit (fun () ->
+    let* sctx = sctx in
+    let* dir_contents = Dir_contents.get sctx ~dir:stanza_dir in
+    let* scope = Scope.DB.find_by_dir stanza_dir in
+    setup_emit_js_rules ~dir_contents ~dir:stanza_dir ~scope ~sctx stanza)
+;;
+
+(* Detect if [dir] is under the target directory of a melange.emit stanza. *)
+let rec under_melange_emit_target ~dir =
+  match Path.Build.parent dir with
+  | None -> Memo.return None
+  | Some parent ->
+    Dune_load.stanzas_in_dir parent
+    >>= (function
+     | None -> under_melange_emit_target ~dir:parent
+     | Some stanzas ->
+       Dune_file.find_stanzas stanzas Melange_stanzas.Emit.key
+       >>| List.find_map ~f:(fun mel ->
+         let target_dir = Melange_stanzas.Emit.target_dir ~dir:parent mel in
+         Option.some_if (Path.Build.equal target_dir dir) mel)
+       >>= (function
+        | None -> under_melange_emit_target ~dir:parent
+        | Some stanza -> Memo.return @@ Some { stanza_dir = parent; stanza }))
+;;
+
+let gen_emit_rules sctx ~dir ({ stanza_dir; stanza } as for_melange) =
+  match Path.Build.equal dir (Melange_stanzas.Emit.target_dir ~dir:stanza_dir stanza) with
+  | false -> Memo.return None
+  | true ->
+    under_melange_emit_target ~dir:stanza_dir
+    >>| (function
+     | None -> Some (emit_rules sctx for_melange)
+     | Some { stanza_dir = parent_melange_emit_dir; stanza = parent_stanza } ->
+       let main_message = Pp.text "melange.emit stanzas cannot be nested" in
+       let annots =
+         let main = User_message.make ~loc:stanza.loc [ main_message ] in
+         let related =
+           [ User_message.make
+               ~loc:parent_stanza.loc
+               [ Pp.text "under this melange stanza" ]
+           ]
+         in
+         User_message.Annots.singleton
+           Compound_user_error.annot
+           [ Compound_user_error.make ~main ~related ]
+       in
+       User_error.raise
+         ~loc:stanza.loc
+         ~annots
+         [ main_message
+         ; Pp.enumerate ~f:Loc.pp_file_colon_line [ parent_stanza.loc; stanza.loc ]
+         ]
+         ~hints:
+           (let emit_dir = Path.Build.drop_build_context_exn stanza_dir in
+            let parent_melange_emit_dir =
+              Path.Build.drop_build_context_exn parent_melange_emit_dir
+            in
+            [ Pp.textf
+                "Move the melange.emit stanza from %s to at least the level of %s"
+                (Path.Source.to_string emit_dir)
+                (Path.Source.to_string parent_melange_emit_dir)
+            ]))
+;;
+
+module Gen_rules = Import.Build_config.Gen_rules
+
+let setup_emit_js_rules sctx ~dir =
+  under_melange_emit_target ~dir
+  >>= function
+  | Some melange ->
+    gen_emit_rules sctx ~dir melange
+    >>| (function
+     | None -> Gen_rules.redirect_to_parent Gen_rules.Rules.empty
+     | Some melange -> Gen_rules.make melange)
+  | None ->
+    (* this should probably be handled by [Dir_status] *)
+    Dune_load.stanzas_in_dir dir
+    >>= (function
+     | None -> Memo.return Gen_rules.no_rules
+     | Some dune_file ->
+       let+ build_dir_only_sub_dirs =
+         Dune_file.find_stanzas dune_file Melange_stanzas.Emit.key
+         >>| List.map ~f:(fun (mel : Melange_stanzas.Emit.t) -> mel.target)
+         >>| Subdir_set.of_list
+         >>| Gen_rules.Build_only_sub_dirs.singleton ~dir
+       in
+       Gen_rules.make ~build_dir_only_sub_dirs (Memo.return Rules.empty))
+;;
