@@ -51,10 +51,9 @@ module Processed = struct
     ; src_dirs : Path.Set.t
     ; flags : string list
     ; extensions : string option Ml_kind.Dict.t list
-    ; melc_flags : string list
     }
 
-  let dyn_of_config { stdlib_dir; obj_dirs; src_dirs; flags; extensions; melc_flags } =
+  let dyn_of_config { stdlib_dir; obj_dirs; src_dirs; flags; extensions } =
     let open Dyn in
     record
       [ "stdlib_dir", option Path.to_dyn stdlib_dir
@@ -62,7 +61,6 @@ module Processed = struct
       ; "src_dirs", Path.Set.to_dyn src_dirs
       ; "flags", list string flags
       ; "extensions", list (Ml_kind.Dict.to_dyn (Dyn.option string)) extensions
-      ; "melc_flags", list string melc_flags
       ]
   ;;
 
@@ -106,7 +104,6 @@ module Processed = struct
           ; src_dirs = Path.Set.empty
           ; flags = [ "-x" ]
           ; extensions = [ { Ml_kind.Dict.intf = None; impl = Some "ext" } ]
-          ; melc_flags = [ "-y" ]
           }
       ; per_module_config = Path.Build.Map.empty
       ; pp_config =
@@ -133,7 +130,7 @@ module Processed = struct
       Error
         "The current Merlin configuration has been generated by another, incompatible, \
          version of Dune. Please rebuild the project. (Using the same version of Dune as \
-         the one running the `ocaml-merlin` server.)"
+         the one running the 'ocaml-merlin' server.)"
   ;;
 
   let serialize_path = Path.to_absolute_filename
@@ -146,8 +143,7 @@ module Processed = struct
     | None, None -> None
   ;;
 
-  let to_sexp ~opens ~pp { stdlib_dir; obj_dirs; src_dirs; flags; extensions; melc_flags }
-    =
+  let to_sexp ~opens ~pp { stdlib_dir; obj_dirs; src_dirs; flags; extensions } =
     let make_directive tag value = Sexp.List [ Atom tag; value ] in
     let make_directive_of_path tag path =
       make_directive tag (Sexp.Atom (serialize_path path))
@@ -167,13 +163,6 @@ module Processed = struct
         | flags ->
           [ make_directive "FLG" (Sexp.List (List.map ~f:(fun s -> Sexp.Atom s) flags)) ]
       in
-      let flags =
-        match melc_flags with
-        | [] -> flags
-        | melc_flags ->
-          make_directive "FLG" (Sexp.List (List.map ~f:(fun s -> Sexp.Atom s) melc_flags))
-          :: flags
-      in
       let flags =
         match pp with
         | None -> flags
@@ -186,9 +175,7 @@ module Processed = struct
       | opens ->
         make_directive
           "FLG"
-          (Sexp.List
-             (List.concat_map opens ~f:(fun name ->
-                [ Sexp.Atom "-open"; Atom (Module_name.to_string name) ])))
+          (Sexp.List (Ocaml_flags.open_flags opens |> List.map ~f:(fun x -> Sexp.Atom x)))
         :: flags
     in
     let suffixes =
@@ -264,17 +251,18 @@ module Processed = struct
     match load_file path with
     | Error msg -> Printf.eprintf "%s\n" msg
     | Ok { per_module_config; pp_config; config } ->
-      let pp_one { module_; opens } =
+      let pp_one (source, { module_; opens }) =
         let open Pp.O in
         let name = Module.name module_ in
         let pp = Module_name.Per_item.get pp_config name in
         let sexp = to_sexp ~opens ~pp config in
-        Pp.vbox (Pp.text (Module_name.to_string name))
+        Pp.hvbox
+          (Pp.textf "%s: %s" (Module_name.to_string name) (Path.Build.to_string source))
         ++ Pp.newline
         ++ Pp.vbox (Sexp.pp sexp)
       in
       let pp =
-        Path.Build.Map.values per_module_config
+        Path.Build.Map.to_list per_module_config
         |> Pp.concat_map ~sep:Pp.cut ~f:pp_one
         |> Pp.vbox
       in
@@ -286,7 +274,7 @@ module Processed = struct
     | Error msg -> Printf.eprintf "%s\n" msg
     | Ok [] -> Printf.eprintf "No merlin configuration found.\n"
     | Ok (init :: tl) ->
-      let pp_configs, obj_dirs, src_dirs, flags, extensions, melc_flags =
+      let pp_configs, obj_dirs, src_dirs, flags, extensions =
         (* We merge what is easy to merge and ignore the rest *)
         List.fold_left
           tl
@@ -295,30 +283,20 @@ module Processed = struct
             , init.config.obj_dirs
             , init.config.src_dirs
             , [ init.config.flags ]
-            , init.config.extensions
-            , init.config.melc_flags )
+            , init.config.extensions )
           ~f:
             (fun
-              (acc_pp, acc_obj, acc_src, acc_flags, acc_ext, acc_melc_flags)
+              (acc_pp, acc_obj, acc_src, acc_flags, acc_ext)
               { per_module_config = _
               ; pp_config
-              ; config =
-                  { stdlib_dir = _; obj_dirs; src_dirs; flags; extensions; melc_flags }
+              ; config = { stdlib_dir = _; obj_dirs; src_dirs; flags; extensions }
               }
             ->
             ( pp_config :: acc_pp
             , Path.Set.union acc_obj obj_dirs
             , Path.Set.union acc_src src_dirs
             , flags :: acc_flags
-            , extensions @ acc_ext
-            , match acc_melc_flags with
-              | [] -> melc_flags
-              | acc_melc_flags -> acc_melc_flags ))
-      in
-      let flags =
-        match melc_flags with
-        | [] -> flags
-        | melc -> melc :: flags
+            , extensions @ acc_ext ))
       in
       Printf.printf
         "%s\n"
@@ -421,16 +399,21 @@ module Unprocessed = struct
   let pp_flag_of_action ~expander ~loc ~action : Processed.pp_flag option Action_builder.t
     =
     match (action : Dune_lang.Action.t) with
-    | Run (exe, args) ->
+    | Run args ->
       (match
          let open Option.O in
          let* args, input_file = List.destruct_last args in
-         if String_with_vars.is_pform input_file (Var Input_file) then Some args else None
+         match input_file with
+         | Slang.Literal input_file ->
+           if String_with_vars.is_pform input_file (Var Input_file)
+           then Some args
+           else None
+         | _ -> None
        with
        | None -> Action_builder.return None
        | Some args ->
          let action =
-           let action = Action_unexpanded.Run (exe, args) in
+           let action = Action_unexpanded.Run args in
            let chdir = Expander.context expander |> Context.build_dir in
            Action_unexpanded.expand_no_targets
              ~loc
@@ -459,23 +442,22 @@ module Unprocessed = struct
     | _ -> Action_builder.return None
   ;;
 
-  let pp_flags sctx ~expander lib_name preprocess
+  let pp_flags ctx ~expander lib_name preprocess
     : Processed.pp_flag option Action_builder.t
     =
-    match
-      Preprocess.remove_future_syntax
-        preprocess
-        ~for_:Merlin
-        (Super_context.context sctx |> Context.ocaml).version
-    with
+    let open Action_builder.O in
+    let* ocaml = Action_builder.of_memo (Context.ocaml ctx) in
+    match Preprocess.remove_future_syntax preprocess ~for_:Merlin ocaml.version with
     | Action (loc, (action : Dune_lang.Action.t)) ->
       pp_flag_of_action ~expander ~loc ~action
     | No_preprocessing -> Action_builder.return None
     | Pps { loc; pps; flags; staged = _ } ->
       let open Action_builder.O in
       let+ exe, flags =
-        let scope = Expander.scope expander in
-        Preprocessing.get_ppx_driver sctx ~loc ~expander ~lib_name ~flags ~scope pps
+        let* scope =
+          Expander.dir expander |> Scope.DB.find_by_dir |> Action_builder.of_memo
+        in
+        Preprocessing.get_ppx_driver ctx ~loc ~expander ~lib_name ~flags ~scope pps
       in
       let args = encode_command ~bin:(Path.build exe) ~args:("--as-ppx" :: flags) in
       Some { Processed.flag = Processed.Pp_kind.Ppx; args }
@@ -498,10 +480,10 @@ module Unprocessed = struct
   module Per_item_action_builder =
     Module_name.Per_item.Make_monad_traversals (Action_builder)
 
-  let pp_config t sctx ~expander =
+  let pp_config t ctx ~expander =
     Per_item_action_builder.map
       t.config.preprocess
-      ~f:(pp_flags sctx ~expander t.config.libname)
+      ~f:(pp_flags ctx ~expander t.config.libname)
   ;;
 
   let process
@@ -544,7 +526,7 @@ module Unprocessed = struct
         | Melange ->
           Action_builder.of_memo
             (let open Memo.O in
-             let scope = Expander.scope expander in
+             let* scope = Scope.DB.find_by_dir (Expander.dir expander) in
              let libs = Scope.libs scope in
              Lib.DB.find libs (Lib_name.of_string "melange")
              >>= function
@@ -562,8 +544,8 @@ module Unprocessed = struct
                Lib.Set.union requires (Lib.Set.of_list libs)
              | None -> Memo.return requires)
       in
-      let* flags = flags
-      and* src_dirs, obj_dirs =
+      let+ flags = flags
+      and+ src_dirs, obj_dirs =
         Action_builder.of_memo
           (let open Memo.O in
            Memo.parallel_map (Lib.Set.to_list requires) ~f:(fun lib ->
@@ -582,20 +564,8 @@ module Unprocessed = struct
       let src_dirs =
         Path.Set.union src_dirs (Path.Set.of_list_map ~f:Path.source more_src_dirs)
       in
-      let+ melc_flags =
-        match t.config.mode with
-        | Ocaml _ -> Action_builder.return []
-        | Melange ->
-          let+ melc_compiler =
-            Action_builder.of_memo (Melange_binary.melc sctx ~loc:None ~dir)
-          in
-          (match melc_compiler with
-           | Error _ -> []
-           | Ok path ->
-             [ Processed.Pp_kind.to_flag Ppx; Processed.serialize_path path ^ " -as-ppx" ])
-      in
-      { Processed.stdlib_dir; src_dirs; obj_dirs; flags; extensions; melc_flags }
-    and+ pp_config = pp_config t sctx ~expander in
+      { Processed.stdlib_dir; src_dirs; obj_dirs; flags; extensions }
+    and+ pp_config = pp_config t (Super_context.context sctx) ~expander in
     let per_module_config =
       (* And copy for each module the resulting pp flags *)
       Modules.fold_no_vlib modules ~init:[] ~f:(fun m init ->
@@ -605,7 +575,7 @@ module Unprocessed = struct
         |> Path.Build.Set.fold ~init ~f:(fun src acc ->
           let config =
             { Processed.module_ = Module.set_pp m None
-            ; opens = Modules.alias_for modules m |> List.map ~f:Module.name
+            ; opens = Modules.local_open modules m
             }
           in
           (src, config) :: acc))
