@@ -7,23 +7,17 @@ open Import
 
 module Origin : sig
   type t =
-    | Library of Dune_file.Library.t
-    | Executables of Dune_file.Executables.t
+    | Library of Library.t
+    | Executables of Executables.t
     | Melange of Melange_stanzas.Emit.t
 
   val loc : t -> Loc.t
-end
-
-module Artifacts : sig
-  type t
-
-  val lookup_module : t -> Module_name.t -> (Path.Build.t Obj_dir.t * Module.t) option
-  val lookup_library : t -> Lib_name.t -> Lib_info.local option
+  val to_dyn : t -> Dyn.t
 end
 
 type t
 
-val artifacts : t -> Artifacts.t Memo.t
+val artifacts : t -> Artifacts_obj.t Memo.t
 
 type for_ =
   | Library of Lib_name.t (** Library name *)
@@ -38,7 +32,7 @@ val modules_and_obj_dir : t -> for_:for_ -> Modules.t * Path.Build.t Obj_dir.t
 val modules : t -> for_:for_ -> Modules.t
 
 (** Find out the origin of the stanza for a given module *)
-val find_origin : t -> Module_name.t -> Origin.t option
+val find_origin : t -> Module_name.Path.t -> Origin.t option
 
 val empty : t
 
@@ -49,13 +43,17 @@ val empty : t
     all virtual modules are implemented - make sure that we construct [Module.t]
     with the correct [kind] *)
 
+val include_subdirs : t -> Include_subdirs.t
+
 val make
-  :  Dune_file.t
+  :  Stanza.t list
+  -> expander:Expander.t
   -> dir:Path.Build.t
-  -> scope:Scope.t
-  -> lib_config:Lib_config.t
+  -> libs:Lib.DB.t Memo.t
+  -> project:Dune_project.t
+  -> lib_config:Lib_config.t Memo.t
   -> loc:Loc.t
   -> lookup_vlib:(loc:Loc.t -> dir:Path.Build.t -> t Memo.t)
-  -> include_subdirs:Loc.t * Dune_file.Include_subdirs.t
-  -> dirs:(Path.Build.t * string list * Filename.Set.t) list
+  -> include_subdirs:Loc.t * Include_subdirs.t
+  -> dirs:Source_file_dir.t list
   -> t Memo.t
