@@ -14,12 +14,7 @@ let force_read_cmi source_file = [ "-intf-suffix"; Path.extension source_file ]
    generation *)
 
 let opens modules m =
-  match Modules.local_open modules m with
-  | [] -> Command.Args.empty
-  | modules ->
-    Command.Args.S
-      (List.map modules ~f:(fun name ->
-         Command.Args.As [ "-open"; Module_name.to_string name ]))
+  Command.Args.As (Modules.local_open modules m |> Ocaml_flags.open_flags)
 ;;
 
 let other_cm_files ~opaque ~cm_kind ~obj_dir =
@@ -112,6 +107,7 @@ let build_cm
     | _ -> default
   in
   let open Memo.O in
+  let ocaml = Compilation_context.ocaml cctx in
   let* compiler =
     match mode with
     | Melange ->
@@ -120,7 +116,7 @@ let build_cm
       Some melc
     | Ocaml mode ->
       Memo.return
-        (let compiler = Ocaml_toolchain.compiler (Context.ocaml ctx) mode in
+        (let compiler = Ocaml_toolchain.compiler ocaml mode in
          (* TODO one day remove this silly optimization *)
          match compiler with
          | Ok _ as s -> Some s
@@ -132,11 +128,7 @@ let build_cm
    let+ src = Module.file m ~ml_kind in
    let dst = Obj_dir.Module.cm_file_exn obj_dir m ~kind:cm_kind in
    let obj =
-     Obj_dir.Module.obj_file
-       obj_dir
-       m
-       ~kind:(Ocaml Cmx)
-       ~ext:(Context.ocaml ctx).lib_config.ext_obj
+     Obj_dir.Module.obj_file obj_dir m ~kind:(Ocaml Cmx) ~ext:ocaml.lib_config.ext_obj
    in
    let open Memo.O in
    let* extra_args, extra_deps, other_targets =
@@ -204,8 +196,7 @@ let build_cm
    in
    let opaque_arg : _ Command.Args.t =
      let intf_only = cm_kind = Ocaml Cmi && not (Module.has m ~ml_kind:Impl) in
-     if opaque
-        || (intf_only && Ocaml.Version.supports_opaque_for_mli (Context.ocaml ctx).version)
+     if opaque || (intf_only && Ocaml.Version.supports_opaque_for_mli ocaml.version)
      then A "-opaque"
      else Command.Args.empty
    in
@@ -293,9 +284,10 @@ let build_module ?(force_write_cmi = false) ?(precompiled_cmi = false) cctx m =
       let* () = build_cm ~cm_kind:(Ocaml Cmo) ~phase:None
       and* () =
         let ctx = Compilation_context.context cctx in
+        let ocaml = Compilation_context.ocaml cctx in
         let can_split =
-          Ocaml.Version.supports_split_at_emit (Context.ocaml ctx).version
-          || Ocaml_config.is_dev_version (Context.ocaml ctx).ocaml_config
+          Ocaml.Version.supports_split_at_emit ocaml.version
+          || Ocaml_config.is_dev_version ocaml.ocaml_config
         in
         match Context.fdo_target_exe ctx, can_split with
         | None, _ -> build_cm ~cm_kind:(Ocaml Cmx) ~phase:None
@@ -326,7 +318,7 @@ let build_module ?(force_write_cmi = false) ?(precompiled_cmi = false) cctx m =
               ~obj_dir
               ~config:None
           in
-          action_with_targets >>= Super_context.add_rule sctx ~dir))
+          Super_context.add_rule sctx ~dir action_with_targets))
   in
   Memo.when_ melange (fun () ->
     let* () = build_cm ~cm_kind:(Melange Cmj) ~phase:None in
@@ -350,6 +342,7 @@ let ocamlc_i ~deps cctx (m : Module.t) ~output =
   in
   let ocaml_flags = Ocaml_flags.get (Compilation_context.flags cctx) (Ocaml Byte) in
   let modules = Compilation_context.modules cctx in
+  let ocaml = Compilation_context.ocaml cctx in
   Super_context.add_rule
     sctx
     ~dir
@@ -358,7 +351,7 @@ let ocamlc_i ~deps cctx (m : Module.t) ~output =
        (let open Action_builder.With_targets.O in
         Action_builder.with_no_targets cm_deps
         >>> Command.run
-              (Ok (Context.ocaml ctx).ocamlc)
+              (Ok ocaml.ocamlc)
               ~dir:(Path.build (Context.build_dir ctx))
               ~stdout_to:output
               [ Command.Args.dyn ocaml_flags
