@@ -19,14 +19,14 @@ module Modules_data : sig
     }
 end
 
-val deps_of :
-     Modules_data.t
+val deps_of
+  :  Modules_data.t
   -> ml_kind:Ml_kind.t
   -> Module.t
   -> Module.t list Action_builder.t Memo.t
 
-val read_deps_of :
-     obj_dir:Path.Build.t Obj_dir.t
+val read_deps_of
+  :  obj_dir:Path.Build.t Obj_dir.t
   -> modules:Modules.t
   -> ml_kind:Ml_kind.t
   -> Module.t
@@ -36,8 +36,8 @@ val read_deps_of :
     immediate dependencies found in the modules of [modules] for the file with
     kind [ml_kind] of the module [unit]. If there is no such file with kind
     [ml_kind], then an empty list of dependencies is returned. *)
-val read_immediate_deps_of :
-     obj_dir:Path.Build.t Obj_dir.t
+val read_immediate_deps_of
+  :  obj_dir:Path.Build.t Obj_dir.t
   -> modules:Modules.t
   -> ml_kind:Ml_kind.t
   -> Module.t
