@@ -33,7 +33,7 @@ let find_module sctx src =
     let* dir_contents = drop_rules @@ fun () -> Dir_contents.get sctx ~dir in
     let* ocaml = Dir_contents.ocaml dir_contents in
     let stanza =
-      match Ml_sources.find_origin ocaml module_name with
+      match Ml_sources.find_origin ocaml [ module_name ] with
       | Some (Executables exes) -> Some (`Executables exes)
       | Some (Library lib) -> Some (`Library lib)
       | None | Some (Melange _) -> None
@@ -71,7 +71,7 @@ let module_deps cctx module_ =
     Ocaml.Ml_kind.Dict.get dg Impl
   in
   let action = Dep_graph.deps_of dep_graph module_ in
-  let+ graph, _ = Action_builder.run action Eager in
+  let+ graph, _ = Action_builder.evaluate_and_collect_facts action in
   graph
 ;;
 
