@@ -4,22 +4,20 @@ module Source : sig
   type t
 
   val make : dir:Path.Build.t -> loc:Loc.t -> main:string -> name:string -> t
-
   val loc : t -> Loc.t
-
   val modules : t -> Pp_spec.t -> Modules.t Memo.t
-
   val obj_dir : t -> Path.Build.t Obj_dir.t
 end
 
 type t
 
-val setup_rules : t -> unit Memo.t
+val setup_rules : t -> linkage:Exe.Linkage.t -> unit Memo.t
 
-val make :
-     cctx:Compilation_context.t
+val make
+  :  cctx:Compilation_context.t
   -> source:Source.t
   -> preprocess:Preprocess.Without_instrumentation.t Preprocess.t
+  -> Expander.t
   -> t
 
 type directives =
@@ -34,9 +32,9 @@ type directives =
 val print_toplevel_init_file : directives -> unit
 
 module Stanza : sig
-  val setup :
-       sctx:Super_context.t
+  val setup
+    :  sctx:Super_context.t
     -> dir:Path.Build.t
-    -> toplevel:Dune_file.Toplevel.t
+    -> toplevel:Toplevel_stanza.t
     -> unit Memo.t
 end
