@@ -7,11 +7,6 @@ let[@inline] loc_compare_start_pos_cnum a b =
   Int.compare a b
 ;;
 
-type hint =
-  { on : string
-  ; candidates : string list
-  }
-
 module Name = struct
   module T = struct
     type t = string
@@ -252,13 +247,6 @@ let parse t context sexp =
   result ctx (t ctx [ sexp ])
 ;;
 
-let set_input : type k. Ast.t list -> (unit, k) parser =
-  fun sexps context _ ->
-  match context with
-  | Values _ -> (), sexps
-  | Fields _ -> (), Fields.of_values sexps
-;;
-
 let capture ctx state =
   let f t = result ctx (t ctx state) in
   f, []
