@@ -12,7 +12,7 @@ type 'a t =
   }
 
 (** Compute the opam layout from prefix. the opam layout is used for _build *)
-val opam_from_prefix : Path.t -> Path.t t
+val opam_from_prefix : 'a -> relative:('a -> string -> 'a) -> 'a t
 
 (** Some roots (e.g. libexec) have another roots as default (e.g. lib) *)
 val complete : 'a option t -> 'a option t
@@ -23,7 +23,6 @@ val map : f:('a -> 'b) -> 'a t -> 'b t
 val first_has_priority : 'a option t -> 'a option t -> 'a option t
 
 val to_env_without_path : Path.Build.t t -> (Env.Var.t * Path.Build.t) list
-
 val add_to_env : Path.Build.t t -> Env.t -> Env.t
-
 val make : 'a -> relative:('a -> string -> 'a) -> 'a t
+val make_all : 'a -> 'a t
